pub trait OutboundRelayLayer {
// Required method
fn decrypt_outbound(
&mut self,
cmd: ChanCmd,
cell: &mut RelayCellBody,
) -> Option<SendmeTag>;
}Available on crate feature
bench only.Expand description
Represent a relay’s view of the outbound crypto state on a given circuit.
Required Methods
Sourcefn decrypt_outbound(
&mut self,
cmd: ChanCmd,
cell: &mut RelayCellBody,
) -> Option<SendmeTag>
fn decrypt_outbound( &mut self, cmd: ChanCmd, cell: &mut RelayCellBody, ) -> Option<SendmeTag>
Decrypt a RelayCellBody that is coming from the client.
Return an authentication tag if it is addressed to us.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".
Implementors
impl<EtBC, PrfBC> OutboundRelayLayer for tor_proto::bench_utils::cgo::RelayOutbound<EtBC, PrfBC>where
EtBC: BlkCipherEnc,
PrfBC: BlkCipherEnc,
Available on crate feature
counter-galois-onion only.