pub struct VegasParamsBuilder {
cell_in_queue_params: Option<VegasQueueParams>,
ss_cwnd_max: Option<u32>,
cwnd_full_gap: Option<u32>,
cwnd_full_min_pct: Option<Percentage<u32>>,
cwnd_full_per_cwnd: Option<u32>,
}Expand description
Builder for VegasParams.
Fields
cell_in_queue_params: Option<VegasQueueParams>The amount of queued cells that Vegas can tolerate before reacting.
ss_cwnd_max: Option<u32>A hard-max on the congestion window in Slow Start.
cwnd_full_gap: Option<u32>This parameter defines the integer number of ‘cc_sendme_inc’ multiples of gap allowed between inflight and cwnd, to still declare the cwnd full.
cwnd_full_min_pct: Option<Percentage<u32>>This parameter defines a low watermark in percent.
cwnd_full_per_cwnd: Option<u32>This parameter governs how often a cwnd must be full.
Implementations
Sourceimpl VegasParamsBuilder
impl VegasParamsBuilder
Sourcepub fn cell_in_queue_params(&mut self, value: VegasQueueParams) -> &mut Self
pub fn cell_in_queue_params(&mut self, value: VegasQueueParams) -> &mut Self
The amount of queued cells that Vegas can tolerate before reacting.
Sourcepub fn ss_cwnd_max(&mut self, value: u32) -> &mut Self
pub fn ss_cwnd_max(&mut self, value: u32) -> &mut Self
A hard-max on the congestion window in Slow Start.
Sourcepub fn cwnd_full_gap(&mut self, value: u32) -> &mut Self
pub fn cwnd_full_gap(&mut self, value: u32) -> &mut Self
This parameter defines the integer number of ‘cc_sendme_inc’ multiples of gap allowed between inflight and cwnd, to still declare the cwnd full.
Sourcepub fn cwnd_full_min_pct(&mut self, value: Percentage<u32>) -> &mut Self
pub fn cwnd_full_min_pct(&mut self, value: Percentage<u32>) -> &mut Self
This parameter defines a low watermark in percent.
Sourcepub fn cwnd_full_per_cwnd(&mut self, value: u32) -> &mut Self
pub fn cwnd_full_per_cwnd(&mut self, value: u32) -> &mut Self
This parameter governs how often a cwnd must be full.
Sourcepub fn build(&self) -> Result<VegasParams, ConfigBuildError>
pub fn build(&self) -> Result<VegasParams, ConfigBuildError>
Sourcefn create_empty() -> Self
fn create_empty() -> Self
Create an empty builder, with all fields set to None or PhantomData.