pub(super) enum CircEnt {
Opening {
create_response_sender: Sender<CreateResponse>,
cell_sender: Sender<AnyChanMsg, MpscSpec>,
padding_ctrl: PaddingController,
},
OpenOrigin {
cell_sender: Sender<AnyChanMsg, MpscSpec>,
padding_ctrl: PaddingController,
},
OpenRelay {
_circ: Arc<RelayCirc>,
cell_sender: Sender<AnyChanMsg, MpscSpec>,
padding_ctrl: PaddingController,
},
DestroySent(HalfCirc),
}Expand description
An entry in the circuit map. Right now, we only have “here’s the way to send cells to a given circuit”, but that’s likely to change.
Variants
Opening
An origin circuit that has not yet received a CREATED cell.
For this circuit, the CREATED* cell or DESTROY cell gets sent to the oneshot sender to tell the corresponding PendingClientCirc that the handshake is done.
Once that’s done, the CircuitRxSender mpsc sender will be used to send subsequent
cells to the circuit.
Fields
create_response_sender: Sender<CreateResponse>The oneshot sender on which to report a create response
cell_sender: Sender<AnyChanMsg, MpscSpec>A sink which should receive all the relay cells for this circuit from this channel
padding_ctrl: PaddingControllerA padding controller we should use when reporting flushed cells.
OpenOrigin
An origin circuit (a circuit which originated here) that is open and can be given relay cells.
Fields
cell_sender: Sender<AnyChanMsg, MpscSpec>A sink which should receive all the relay cells for this circuit from this channel
padding_ctrl: PaddingControllerA padding controller we should use when reporting flushed cells.
OpenRelay
A relay circuit (a circuit in which we are a hop on the path) that is open and can be given relay cells.
Fields
_circ: Arc<RelayCirc>A handle to the circuit.
TODO(relay): We need to store the Arc<RelayCirc> somewhere
and currently this seems like the best place to store it.
As we implement more functionality maybe we’ll find a better place to store it,
in which case we should consider combining the OpenOrigin and OpenRelay variants.
cell_sender: Sender<AnyChanMsg, MpscSpec>A sink which should receive all the relay cells for this circuit from this channel
padding_ctrl: PaddingControllerA padding controller we should use when reporting flushed cells.
DestroySent(HalfCirc)
A circuit where we have sent a DESTROY, but the other end might not have gotten a DESTROY yet.