#[non_exhaustive]pub enum ChannelType {
ClientInitiator,
RelayInitiator,
RelayResponder {
authenticated: bool,
},
}Expand description
This indicate what type of channel it is. It allows us to decide for the correct channel cell state machines and authentication process (if any).
It is created when a channel is requested for creation which means the subsystem wanting to open a channel needs to know what type it wants.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
ClientInitiator
Client: Initiated from a client to a relay. Client is unauthenticated and relay is authenticated.
RelayInitiator
Relay: Initiating as a relay to a relay. Both sides are authenticated.
RelayResponder
Relay: Responding as a relay to a relay or client. Authenticated or Unauthenticated.
Fields
authenticated: boolIndicate if the channel is authenticated. Responding as a relay can be either from a Relay (authenticated) or a Client/Bridge (Unauthenticated). We only know this information once the handshake is completed.
This side is always authenticated, the other side can be if a relay or not if bridge/client. This is set to false unless we end up authenticating the other side meaning a relay.