pub enum CtrlMsg {
Shutdown,
CloseCircuit(CircId),
AllocateCircuit {
created_sender: Sender<CreateResponse>,
sender: Sender<AnyChanMsg, MpscSpec>,
tx: Sender<Result<(CircId, UniqId, PaddingController, PaddingEventStream)>>,
},
ConfigUpdate(Arc<ChannelPaddingInstructionsUpdates>),
KistConfigUpdate(KistParams),
SetChannelPadder {
padder: Option<CircuitPadder>,
sender: Sender<Result<()>>,
},
}Expand description
A message telling the channel reactor to do something.
Variants
Shutdown
Shut down the reactor.
CloseCircuit(CircId)
Tell the reactor that a given circuit has gone away.
AllocateCircuit
Allocate a new circuit in this channel’s circuit map, generating an ID for it and registering senders for messages received for the circuit.
Fields
created_sender: Sender<CreateResponse>Channel to send the circuit’s CreateResponse down.
sender: Sender<AnyChanMsg, MpscSpec>Channel to send other messages from this circuit down.
ConfigUpdate(Arc<ChannelPaddingInstructionsUpdates>)
Enable/disable/reconfigure channel padding
The sender of these messages is responsible for the optimisation of
ensuring that “no-change” messages are elided.
(This is implemented in ChannelsParamsUpdatesBuilder.)
These updates are done via a control message to avoid adding additional branches to the
main reactor select!.
KistConfigUpdate(KistParams)
Enable/disable/reconfigure KIST.
Like in the case of ConfigUpdate,
the sender of these messages is responsible for the optimisation of
ensuring that “no-change” messages are elided.
SetChannelPadder
circ-padding-manual only.Change the current padding implementation to the one provided.