ChannelCellHandler

Enum ChannelCellHandler 

pub(crate) enum ChannelCellHandler {
    New(NewChannelHandler),
    Handshake(HandshakeChannelHandler),
    Open(OpenChannelHandler),
}
Channel cell handler which is always in three state.

This ALWAYS starts the handler at New. This can only be constructed from a ChannelType which forces it to start at New.

From the New state, it will automatically transition to the right state as information is attached to it (ex: link protocol version).

Variants



New(NewChannelHandler)

When a network connection opens to another endpoint, the channel is considered “New” and so we use this handler to start the handshake.



Handshake(HandshakeChannelHandler)

We opened and negotiated a VERSIONS cell. If successful, we transition to this cell handler with sole purpose to handle the handshake phase.



Open(OpenChannelHandler)

Once the handshake is successful, the channel is Open and we use this handler.

Implementations

Source

impl ChannelCellHandler

pub(crate) fn channel_type(&self) -> ChannelType

Return the ChannelType of the inner handler.

Set link protocol for this channel cell handler. This transition the handler into the handshake handler state.

An error is returned if the current handler is NOT the New one or if the link version is unknown.

pub(crate) fn set_open(&mut self) -> Result<(), ChanError>

This transition into the open handler state.

An error is returned if the current handler is NOT the Handshake one.

pub(crate) fn set_authenticated(&mut self) -> Result<(), ChanError>

Mark this handler as authenticated.

This can only happen during the Handshake process as a New handler can’t be authenticated from the start and an Open handler can only be opened after authentication.

pub(crate) fn take_send_log_digest(&mut self) -> Result<[u8; 32], ChanError>

The digest of bytes sent on this channel.

This should only ever be called once as it consumes the send log.

This will return an error if one of:

  • The channel is not recording the send log.
  • The send log digest has already been taken.
  • This cell handler is not using a handshake handler.
pub(crate) fn take_recv_log_digest(&mut self) -> Result<[u8; 32], ChanError>

The digest of bytes received on this channel.

This should only ever be called once as it consumes the receive log.

This will return None if one of:

  • The channel is not recording the receive log.
  • The receive log digest has already been taken.
  • This cell handler is not using a handshake handler.