pub(crate) struct HandshakeChannelHandler {
filter: MessageFilter,
inner: ChannelCodec,
send_log: Option<Sha256>,
recv_log: Option<Sha256>,
}Expand description
The handshake channel handler which is used to decode and encode cells onto a channel that is handshaking with an endpoint.
Fields
filter: MessageFilterMessage filter used to allow or not a certain message.
inner: ChannelCodecThe cell codec that we’ll use to encode and decode our cells.
send_log: Option<Sha256>The digest of bytes sent on this channel.
Will be used for the SLOG or CLOG of the AUTHENTICATE cell.
recv_log: Option<Sha256>The digest of bytes received on this channel.
Will be used for the SLOG or CLOG of the AUTHENTICATE cell.
Implementations
Sourceimpl HandshakeChannelHandler
impl HandshakeChannelHandler
Sourcefn new(new_handler: &mut NewChannelHandler, link_version: LinkVersion) -> Self
fn new(new_handler: &mut NewChannelHandler, link_version: LinkVersion) -> Self
Constructor
Sourcefn finalize_log(log: Option<Sha256>) -> Option<[u8; 32]>
fn finalize_log(log: Option<Sha256>) -> Option<[u8; 32]>
Internal helper: Take a SHA256 digest and finalize it if any. None is returned if no log digest is given.
Sourcefn next_handler(&mut self) -> OpenChannelHandler
fn next_handler(&mut self) -> OpenChannelHandler
Return an open handshake handler.
Sourcepub(crate) fn take_send_log_digest(&mut self) -> Option<[u8; 32]>
pub(crate) fn take_send_log_digest(&mut self) -> Option<[u8; 32]>
The digest of bytes sent on this channel.
This should only ever be called once as it consumes the send log.
This will return None if one of:
- The channel is not recording the send log.
- The send log digest has already been taken.
Sourcepub(crate) fn take_recv_log_digest(&mut self) -> Option<[u8; 32]>
pub(crate) fn take_recv_log_digest(&mut self) -> Option<[u8; 32]>
The digest of bytes received on this channel.
This should only ever be called once as it consumes the receive log.
This will return None if one of:
- The channel is not recording the receive log.
- The receive log digest has already been taken.
Sourcepub(crate) fn channel_type(&self) -> ChannelType
pub(crate) fn channel_type(&self) -> ChannelType
Return the ChannelType of this handler.
Sourcepub(crate) fn set_authenticated(&mut self)
pub(crate) fn set_authenticated(&mut self)
Mark this handler as authenticated.