pub(crate) enum AuthLogAction {
Leave,
Take,
}Expand description
Helper: This enum is for adding semantic to the function receiving cells indicating it to either take the auth log out or leave it in place.
With this, we avoid using a flat bool which is confusing at the call site.
Variants
Implementations
Sourceimpl AuthLogAction
impl AuthLogAction
Sourcefn is_take(&self) -> bool
fn is_take(&self) -> bool
Return true iff this value is AuthLogAction::Take