pub(crate) fn get_cert(certs: &Certs, tp: CertType) -> Result<KeyUnknownCert>Expand description
Helper: get a cert from our Certs cell, and convert errors appropriately.
pub(crate) fn get_cert(certs: &Certs, tp: CertType) -> Result<KeyUnknownCert>Helper: get a cert from our Certs cell, and convert errors appropriately.