unauthenticated_clock_skew

Function unauthenticated_clock_skew 

pub(crate) fn unauthenticated_clock_skew(
    netinfo_cell: &Netinfo,
    netinfo_rcvd_at: Instant,
    versions_flushed_at: Instant,
    versions_flushed_wallclock: SystemTime,
) -> ClockSkew
Helper: Calculate a clock skew from the msg::Netinfo cell data and the time at which we sent the msg::Versions cell.

This is unauthenticated as in not validated with the certificates. Before using it, make sure that you have authenticated the other party.