verify_link_auth_cert

Function verify_link_auth_cert 

pub(crate) fn verify_link_auth_cert(
    certs: &Certs,
    kp_relaysign_ed: &Ed25519Identity,
    now: Option<SystemTime>,
    clock_skew: ClockSkew,
) -> Result<Ed25519Identity>
Validate the LINK_AUTH cert (CertType 6).

certs is the msg::Certs cell received during the handshake.

kp_relaysign_ed is the relay signing ed25519 key taken from the signing cert IDENTITY_V_SIGNING. It is used to sign the LINK_AUTH cert.

‘now’ is the time at which to check that certificates are valid. None means to use the current time. It can be used for testing to override the current view of the time.

The clock_skew is the time skew detected during the handshake.

If verification is successful, return the peer KP_link_ed.