pub(crate) struct UnverifiedInitiatorChannel<T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider> {
pub(crate) inner: UnverifiedChannel<T, S>,
pub(crate) certs_cell: Certs,
}Expand description
A base initiator channel on which versions have been negotiated and the relay’s handshake has
been read, but where the msg::Certs has not been checked.
Both relay and client have specialized objects for an unverified channel which include this one as the base in order to share functionnalities.
We need this intermediary object between the specialized one (client/relay) and the
UnverifiedChannel because certs validation is quite different from a respodner channel.
This avoid code duplication.
Fields
inner: UnverifiedChannel<T, S>Base unverified channel.
certs_cell: CertsThe msg::Certs received during the handshake.
Implementations
Sourceimpl<T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider> UnverifiedInitiatorChannel<T, S>
impl<T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider> UnverifiedInitiatorChannel<T, S>
Sourcepub(crate) fn verify<U: ChanTarget + ?Sized>(
self,
peer_target: &U,
peer_cert_digest: [u8; 32],
now: Option<SystemTime>,
) -> Result<VerifiedChannel<T, S>>
pub(crate) fn verify<U: ChanTarget + ?Sized>( self, peer_target: &U, peer_cert_digest: [u8; 32], now: Option<SystemTime>, ) -> Result<VerifiedChannel<T, S>>
Validate all relay identities and TLS cert SIGNING_V_TLS_CERT located in the received
certs_cell.
peer_target is the relay we want to connect to.
‘peer_cert_digest’ is the digest of the x.509 certificate that the peer presented during its TLS handshake (ServerHello).
‘now’ is the time at which to check that certificates are valid. None means to use the
current time. It can be used for testing to override the current view of the time.