VerifiedChannel

Struct VerifiedChannel 

pub(crate) struct VerifiedChannel<T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider> {
    pub(crate) sleep_prov: S,
    pub(crate) memquota: ChannelAccount,
    pub(crate) link_protocol: u16,
    pub(crate) framed_tls: Framed<T, ChannelCellHandler>,
    pub(crate) target_method: Option<ChannelMethod>,
    pub(crate) unique_id: UniqId,
    pub(crate) clock_skew: ClockSkew,
    pub(crate) peer_relay_ids: RelayIds,
    pub(crate) peer_rsa_id_digest: [u8; 32],
}
A base channel on which versions have been negotiated, relay’s handshake has been read, but the client has not yet finished the handshake.

This type is separate from UnverifiedChannel, since finishing the handshake requires a bunch of CPU, and you might want to do it as a separate task or after a yield.

Both relay and client have specialized objects for an unverified channel which include this one as the base in order to share functionalities.

Fields

sleep_prov: S

Runtime handle (insofar as we need it)

memquota: ChannelAccount

Memory quota account

link_protocol: u16

The negotiated link protocol.

framed_tls: Framed<T, ChannelCellHandler>

The Source+Sink on which we’re reading and writing cells.

target_method: Option<ChannelMethod>

Declared target method for this stream, if any.

unique_id: UniqId

Logging identifier for this stream. (Used for logging only.)

clock_skew: ClockSkew

Authenticated clock skew for this peer.

peer_relay_ids: RelayIds

Verified peer identities

peer_rsa_id_digest: [u8; 32]

Validated RSA identity digest of the DER format for this peer.

Implementations

Source

impl<T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider> VerifiedChannel<T, S>

pub(crate) fn set_authenticated(&mut self) -> Result<()>

Mark this channel as authenticated.

pub(crate) fn relay_ids(&self) -> &RelayIds

Return our RelayIds corresponding to this channel identities.

pub(crate) async fn finish( self, peer_netinfo: &Netinfo, my_addrs: &[IpAddr], peer_info: MaybeSensitive<PeerInfo>, channel_mode: ChannelMode, ) -> Result<(Arc<Channel>, Reactor<S>)>
where S: Runtime,

Finalize this channel into an actual channel and its reactor.

peer_netinfo is the received msg::Netinfo cell from the peer.

my_addrs are our advertised addresses as a relay.

peer_info is the verified peer information (identities and addresses).