pub(super) enum OpenChanMsgR2R {
CreateFast(CreateFast),
CreatedFast(CreatedFast),
Create2(Create2),
Created2(Created2),
Destroy(Destroy),
Padding(Padding),
Vpadding(Vpadding),
Relay(Relay),
RelayEarly(RelayEarly),
}Expand description
A channel message that we allow to be sent (bidirectionally) from a Relay to a Relay on an open channel.
Variants
CreateFast(CreateFast)
CreatedFast(CreatedFast)
Create2(Create2)
Created2(Created2)
Destroy(Destroy)
Padding(Padding)
Vpadding(Vpadding)
Relay(Relay)
RelayEarly(RelayEarly)