pub(super) struct MessageFilter {
link_version: LinkVersion,
channel_type: ChannelType,
stage: MessageStage,
}Expand description
A message filter object which is used to learn if a certain message is allowed or not on a channel.
It is pinned to a link protocol version, a channel type and a channel message stage.
Fields
link_version: LinkVersionFor what link protocol version this filter applies for.
channel_type: ChannelTypeFor which channel type this filter applies for.
stage: MessageStageAt which stage this filter applies for.
Implementations
Sourceimpl MessageFilter
impl MessageFilter
Sourcepub(super) fn new(
link_version: LinkVersion,
channel_type: ChannelType,
stage: MessageStage,
) -> Self
pub(super) fn new( link_version: LinkVersion, channel_type: ChannelType, stage: MessageStage, ) -> Self
Constructor
Sourcepub(super) fn channel_type(&self) -> ChannelType
pub(super) fn channel_type(&self) -> ChannelType
Return the ChannelType of this filter.
Sourcepub(super) fn channel_type_mut(&mut self) -> &mut ChannelType
pub(super) fn channel_type_mut(&mut self) -> &mut ChannelType
Return the ChannelType of this filter as a mutable.
Sourcepub(super) fn decode_cell(
&self,
codec: &mut ChannelCodec,
src: &mut BytesMut,
) -> Result<Option<AnyChanCell>, Error>
pub(super) fn decode_cell( &self, codec: &mut ChannelCodec, src: &mut BytesMut, ) -> Result<Option<AnyChanCell>, Error>
Decode a cell from the given bytes for the right link version, channel type and message stage using the codec given.
Sourcepub(super) fn encode_cell(
&self,
cell: AnyChanCell,
codec: &mut ChannelCodec,
dst: &mut BytesMut,
) -> Result<(), Error>
pub(super) fn encode_cell( &self, cell: AnyChanCell, codec: &mut ChannelCodec, dst: &mut BytesMut, ) -> Result<(), Error>
Decode a cell from the given bytes for the right link version, channel type and message stage using the codec given.