type CellRx = CountingStream<Receiver<(AnyChanCell, Option<QueuedCellPaddingInfo>), MpscSpec>>;Expand description
Implementation type for a cell queue held by a reactor.
Aliased Type
struct CellRx { /* private fields */ }type CellRx = CountingStream<Receiver<(AnyChanCell, Option<QueuedCellPaddingInfo>), MpscSpec>>;Implementation type for a cell queue held by a reactor.
struct CellRx { /* private fields */ }