pub(crate) struct CircuitRxSender(Option<CircuitRxSenderInner>);Expand description
The sending end of the SPSC queue for inbound data on its way from channel to circuit
A CircuitRxSender sender is closed for sending as soon as the first
DESTROY message is sent, and will discard any unflushed cells
from its underlying mq_queue, by dropping it.
No Sink implementation
This type intentionally does not implement Sink.
Instead it provides a send() function
similar to SinkExt::send.
The reason for doing it this way is because we cannot provide
a correct Sink::poll:ready() implementation
that wouldn’t block DESTROY cells from being sent
when our underlying MPSC sender is full:
SinkExt::send() calls poll_ready() followed by start_send(),
so in order for our poll_ready() implementation to not block DESTROY
on the MPSC queue’s readiness, it would need to know whether
the cell that will be sent via start_send() is a DESTROY or not,
but that’s not possible because of the way the Sink/SinkExt traits
are designed.
Tuple Fields
0: Option<CircuitRxSenderInner>Implementations
Sourceimpl CircuitRxSender
impl CircuitRxSender
Sourcepub(crate) async fn send(&mut self, msg: AnyChanMsg) -> Result<(), SendError>
pub(crate) async fn send(&mut self, msg: AnyChanMsg) -> Result<(), SendError>
Send a cell down this channel
If the sender is already closed (i.e., if we have already sent DESTROY), this will return an error.
Sourcefn borrow_for_sending(&mut self) -> Result<&mut CircuitRxSenderInner, SendError>
fn borrow_for_sending(&mut self) -> Result<&mut CircuitRxSenderInner, SendError>
Borrow the CircuitRxSenderInner state for sending.
Returns an error if the sender is closed.
Sourcefn take_inner(&mut self) -> Result<CircuitRxSenderInner, SendError>
fn take_inner(&mut self) -> Result<CircuitRxSenderInner, SendError>
Take the inner CircuitRxSenderInner, closing the sender.
Returns an error if the sender is already closed.