fn cvt(limit: u32) -> NonZeroU32Expand description
Convert a limit from the form used in a HopSettings to that used here. (The format we use here is more compact.)
fn cvt(limit: u32) -> NonZeroU32Convert a limit from the form used in a HopSettings to that used here. (The format we use here is more compact.)