pub(crate) struct CircHopInbound {
decoder: RelayCellDecoder,
n_incoming_cells_permitted: Option<NonZeroU32>,
}Expand description
The inbound state of a hop.
Fields
decoder: RelayCellDecoderDecodes relay cells received from this hop.
n_incoming_cells_permitted: Option<NonZeroU32>Remaining permitted incoming relay cells from this hop, plus 1.
(In other words, None represents no limit,
Some(1) represents an exhausted limit,
and Some(n) means that n-1 more cells may be received.)
If this ever decrements from Some(1), then the circuit must be torn down with an error.
Implementations
Sourceimpl CircHopInbound
impl CircHopInbound
Sourcepub(crate) fn new(decoder: RelayCellDecoder, settings: &HopSettings) -> Self
pub(crate) fn new(decoder: RelayCellDecoder, settings: &HopSettings) -> Self
Create a new CircHopInbound.
Sourcepub(crate) fn decode(
&mut self,
cell: BoxedCellBody,
) -> Result<RelayCellDecoderResult>
pub(crate) fn decode( &mut self, cell: BoxedCellBody, ) -> Result<RelayCellDecoderResult>
Parse a RELAY or RELAY_EARLY cell body.
Requires that the cryptographic checks on the message have already been performed
Sourcepub(crate) fn decrement_cell_limit(&mut self) -> Result<()>
pub(crate) fn decrement_cell_limit(&mut self) -> Result<()>
Decrement the limit of inbound cells that may be received from this hop; give an error if it would reach zero.