pub(crate) struct HopSettings {
pub(crate) ccontrol: CongestionControlParams,
pub(crate) flow_ctrl_params: FlowCtrlParameters,
pub(crate) n_incoming_cells_permitted: Option<u32>,
pub(crate) n_outgoing_cells_permitted: Option<u32>,
relay_crypt_protocol: RelayCryptLayerProtocol,
}Expand description
The settings we use for single hop of a circuit.
Unlike CircParameters, this type is crate-internal.
We construct it based on our settings from the circuit,
and from the hop’s actual capabilities.
Then, we negotiate with the hop as part of circuit
creation/extension to determine the actual settings that will be in use.
Finally, we use those settings to construct the negotiated circuit hop.
Fields
ccontrol: CongestionControlParamsThe negotiated congestion control settings for this hop .
flow_ctrl_params: FlowCtrlParametersFlow control parameters that will be used for streams on this hop.
n_incoming_cells_permitted: Option<u32>Maximum number of permitted incoming relay cells for this hop.
n_outgoing_cells_permitted: Option<u32>Maximum number of permitted outgoing relay cells for this hop.
relay_crypt_protocol: RelayCryptLayerProtocolThe relay cell encryption algorithm and cell format for this hop.
Implementations
Sourceimpl HopSettings
impl HopSettings
Sourcepub(crate) fn from_params_and_caps(
hoptype: HopNegotiationType,
params: &CircParameters,
caps: &Protocols,
) -> Result<Self>
pub(crate) fn from_params_and_caps( hoptype: HopNegotiationType, params: &CircParameters, caps: &Protocols, ) -> Result<Self>
Construct a new HopSettings based on params (a set of circuit parameters)
and caps (a set of protocol capabilities for a circuit target).
The resulting settings will represent what the client would prefer to negotiate
(determined by params),
as modified by what the target relay is believed to support (represented by caps).
This represents the HopSettings in a pre-negotiation state:
the circuit negotiation process will modify it.
Sourcepub(crate) fn relay_crypt_protocol(&self) -> RelayCryptLayerProtocol
pub(crate) fn relay_crypt_protocol(&self) -> RelayCryptLayerProtocol
Return the negotiated relay crypto protocol.
Sourcepub(crate) fn circuit_request_extensions(&self) -> Result<Vec<CircRequestExt>>
pub(crate) fn circuit_request_extensions(&self) -> Result<Vec<CircRequestExt>>
Return the client circuit-creation extensions that we should use in order to negotiate these circuit hop parameters.