CreateHandshakeWrap

Trait CreateHandshakeWrap 

pub(crate) trait CreateHandshakeWrap {
    // Required methods
    fn to_chanmsg(&self, bytes: Vec<u8>) -> AnyChanMsg;
    fn decode_chanmsg(&self, msg: CreateResponse) -> Result<Vec<u8>>;
}
An object that can put a given handshake into a ChanMsg for a CREATE* cell, and unwrap a CREATED* cell.

Required Methods

fn to_chanmsg(&self, bytes: Vec<u8>) -> AnyChanMsg

Construct an appropriate ChanMsg to hold this kind of handshake.

fn decode_chanmsg(&self, msg: CreateResponse) -> Result<Vec<u8>>

Decode a ChanMsg to an appropriate handshake value, checking its type.

Implementors