pub(crate) struct CircHopList {
hops: SmallVec<[CircHop; 6]>,
}Expand description
Represents the reactor’s view of a circuit’s hops.
Fields
hops: SmallVec<[CircHop; 6]>The list of hops.
Relays have only one. Clients have one entry per circuit hop.
Implementations
Sourceimpl CircHopList
impl CircHopList
Sourcepub(crate) fn get(&self, hop: Option<HopNum>) -> Option<&CircHop>
pub(crate) fn get(&self, hop: Option<HopNum>) -> Option<&CircHop>
Return a reference to the hop corresponding to hopnum, if there is one.
Relays pass None for the hopnum.
Sourcepub(crate) fn get_mut(&mut self, hop: Option<HopNum>) -> Option<&mut CircHop>
pub(crate) fn get_mut(&mut self, hop: Option<HopNum>) -> Option<&mut CircHop>
Return a mutable reference to the hop corresponding to hopnum, if there is one.
Relays pass None for the hopnum.