CtrlCmd

Enum CtrlCmd 

pub(crate) enum CtrlCmd<C> {
    AwaitStreamRequests {
        incoming_sender: Sender<StreamReqInfo, MpscSpec>,
        cmd_checker: Box<dyn CmdChecker + Send + 'static>,
        done: Sender<Result<()>>,
        hop: Option<HopNum>,
        filter: Box<dyn IncomingStreamRequestFilter>,
    },
    ClosePendingStream {
        hop: Option<HopNum>,
        stream_id: StreamId,
        message: CloseStreamBehavior,
        done: Sender<Result<()>>,
    },
    Custom(C),
}
A control command aimed at the generic forward reactor.

Variants



AwaitStreamRequests

Begin accepting streams on this circuit.

Fields

incoming_sender: Sender<StreamReqInfo, MpscSpec>

A channel for sending information about an incoming stream request.

cmd_checker: Box<dyn CmdChecker + Send + 'static>

A CmdChecker to keep track of which message types are acceptable.

done: Sender<Result<()>>

Oneshot channel to notify on completion.

hop: Option<HopNum>

The hop that is allowed to create streams.

Set to None if we are a relay wanting to accept stream requests.

filter: Box<dyn IncomingStreamRequestFilter>

A filter used to check requests before passing them on.



ClosePendingStream

Close the specified pending incoming stream, sending the provided END message.

A stream is said to be pending if the message for initiating the stream was received but not has not been responded to yet.

This should be used by responders for closing pending incoming streams initiated by the other party on the circuit.

TODO(dedup): this is almost identical to the ClosePendingStream control message from the client-side. We can get rid of the duplication by rewriting the client circuit reactor to use the new multi-reactor architecture

Fields

hop: Option<HopNum>

The hop number the stream is on.

Set to None if we are a relay.

stream_id: StreamId

The stream ID to send the END for.

message: CloseStreamBehavior

The END message to send, if any.

done: Sender<Result<()>>

Oneshot channel to notify on completion.



Custom(C)

An implementation-dependent control command.