pub(crate) struct Reactor<R: Runtime, F: ForwardHandler, B: BackwardHandler> {
unique_id: UniqId,
forward: Option<ForwardReactor<R, F>>,
backward: Option<BackwardReactor<B>>,
control: UnboundedReceiver<CtrlMsg<F::CtrlMsg, B::CtrlMsg>>,
command: UnboundedReceiver<CtrlCmd<F::CtrlCmd, B::CtrlCmd>>,
fwd_ctrl: ReactorCtrl<CtrlCmd<F::CtrlCmd>, CtrlMsg<F::CtrlMsg>>,
bwd_ctrl: ReactorCtrl<CtrlCmd<B::CtrlCmd>, CtrlMsg<B::CtrlMsg>>,
}Expand description
The entry point of the circuit reactor subsystem.
Fields
unique_id: UniqIdThe process-unique identifier of this circuit.
Used for logging.
forward: Option<ForwardReactor<R, F>>The reactor for handling
- cells moving in the forward direction (from the client towards exit), if we are a relay
- incoming cells (coming from the guard), if we are a client
Optional so we can move it out of self in run().
backward: Option<BackwardReactor<B>>The reactor for handling
- cells moving in the backward direction (from the exit towards client), if we are a relay
- outgoing cells (moving towards the guard), if we are a client
Optional so we can move it out of self in run().
control: UnboundedReceiver<CtrlMsg<F::CtrlMsg, B::CtrlMsg>>Receiver for control messages for this reactor, sent by reactor handle objects.
command: UnboundedReceiver<CtrlCmd<F::CtrlCmd, B::CtrlCmd>>Receiver for command messages for this reactor, sent by reactor handle objects.
This MPSC channel is polled in run.
NOTE: this is a separate channel from control, because some messages
have higher priority and need to be handled even if the inbound_chan_tx is not
ready (whereas control messages are not read until the inbound_chan_tx sink
is ready to accept cells).
fwd_ctrl: ReactorCtrl<CtrlCmd<F::CtrlCmd>, CtrlMsg<F::CtrlMsg>>Control channels for the ForwardReactor.
Handles CtrlCmd::Forward and CtrlMsg::Forward messages.
bwd_ctrl: ReactorCtrl<CtrlCmd<B::CtrlCmd>, CtrlMsg<B::CtrlMsg>>Control channels for the BackwardReactor.
Handles CtrlCmd::Backward and CtrlMsg::Backward messages.
Implementations
Sourceimpl<R: Runtime, F: ForwardHandler, B: BackwardHandler> Reactor<R, F, B>
impl<R: Runtime, F: ForwardHandler, B: BackwardHandler> Reactor<R, F, B>
Sourceimpl<R: Runtime, F: ForwardHandler + ControlHandler, B: BackwardHandler + ControlHandler> Reactor<R, F, B>
impl<R: Runtime, F: ForwardHandler + ControlHandler, B: BackwardHandler + ControlHandler> Reactor<R, F, B>
Sourcepub(crate) fn new(
runtime: R,
channel: &Arc<Channel>,
circ_id: CircId,
unique_id: UniqId,
inbound_chan_rx: Receiver<AnyChanMsg, MpscSpec>,
forward_impl: F,
backward_impl: B,
hop_mgr: HopMgr<R>,
padding_ctrl: PaddingController,
padding_event_stream: PaddingEventStream,
bwd_rx: Receiver<ReadyStreamMsg>,
fwd_events: Receiver<F::CircEvent>,
memquota: &CircuitAccount,
) -> (Self, CircReactorHandle<F, B>)
pub(crate) fn new( runtime: R, channel: &Arc<Channel>, circ_id: CircId, unique_id: UniqId, inbound_chan_rx: Receiver<AnyChanMsg, MpscSpec>, forward_impl: F, backward_impl: B, hop_mgr: HopMgr<R>, padding_ctrl: PaddingController, padding_event_stream: PaddingEventStream, bwd_rx: Receiver<ReadyStreamMsg>, fwd_events: Receiver<F::CircEvent>, memquota: &CircuitAccount, ) -> (Self, CircReactorHandle<F, B>)
Create a new circuit reactor.
The reactor will send outbound messages on channel, receive incoming
messages on inbound_chan_rx, and identify this circuit by the channel-local
CircId provided.
The internal unique identifier for this circuit will be unique_id.
Sourcefn handle_shutdown(&self) -> StdResult<(), ReactorError>
fn handle_shutdown(&self) -> StdResult<(), ReactorError>
Handle a shutdown request.
Sourcefn handle_command(
&mut self,
cmd: CtrlCmd<F::CtrlCmd, B::CtrlCmd>,
) -> StdResult<(), ReactorError>
fn handle_command( &mut self, cmd: CtrlCmd<F::CtrlCmd, B::CtrlCmd>, ) -> StdResult<(), ReactorError>
Handle a CtrlCmd.
Sourcefn handle_control(
&mut self,
cmd: CtrlMsg<F::CtrlMsg, B::CtrlMsg>,
) -> StdResult<(), ReactorError>
fn handle_control( &mut self, cmd: CtrlMsg<F::CtrlMsg, B::CtrlMsg>, ) -> StdResult<(), ReactorError>
Handle a CtrlMsg.