ControlHandler

Trait ControlHandler 

pub(crate) trait ControlHandler {
    type CtrlMsg;
    type CtrlCmd;

    // Required methods
    fn handle_cmd(&mut self, cmd: Self::CtrlCmd) -> StdResult<(), ReactorError>;
    fn handle_msg(&mut self, msg: Self::CtrlMsg) -> StdResult<(), ReactorError>;
}
Trait implemented by types that can handle control messages and commands.

Required Associated Types

type CtrlMsg

The type of control message expected by the forward reactor.

type CtrlCmd

The type of control command expected by the forward reactor.

Required Methods

fn handle_cmd(&mut self, cmd: Self::CtrlCmd) -> StdResult<(), ReactorError>

Handle a control command.

fn handle_msg(&mut self, msg: Self::CtrlMsg) -> StdResult<(), ReactorError>

Handle a control message.

Implementors