pub(crate) type CircuitRxReceiver = Receiver<AnyChanMsg, MpscSpec>;Expand description
MPSC queue for inbound data on its way from channel to circuit, receiver
Aliased Type
pub(crate) struct CircuitRxReceiver { /* private fields */ }pub(crate) type CircuitRxReceiver = Receiver<AnyChanMsg, MpscSpec>;MPSC queue for inbound data on its way from channel to circuit, receiver
pub(crate) struct CircuitRxReceiver { /* private fields */ }