CircuitRxSender

Type Alias CircuitRxSender 

pub(crate) type CircuitRxSender = Sender<AnyChanMsg, MpscSpec>;
The following two MPSCs take any channel message as the receiving end can be either a client or a relay circuit reactor. The reactor itself will convert into its restricted message set. On error, the circuit will shutdown as it will be considered a protocol violation.

MPSC queue for inbound data on its way from channel to circuit, sender

Aliased Type

pub(crate) struct CircuitRxSender { /* private fields */ }