#[non_exhaustive]pub struct ClientChannelBuilder {
target: Option<ChannelMethod>,
}Expand description
Structure for building and launching a client Tor channel.
Fields (Non-exhaustive)
This struct is marked as non-exhaustive
Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.target: Option<ChannelMethod>If present, a description of the address we’re trying to connect to, and the way in which we are trying to connect to it.
TODO: at some point, check this against the addresses in the netinfo cell too.
Implementations
Sourceimpl ClientChannelBuilder
impl ClientChannelBuilder
Sourcepub fn set_declared_method(&mut self, target: ChannelMethod)
pub fn set_declared_method(&mut self, target: ChannelMethod)
Set the declared target method of this channel.
Note that nothing enforces the correctness of this method: it doesn’t have to match the real method used to create the TLS stream.
Sourcepub fn launch<T, S>(
self,
tls: T,
sleep_prov: S,
memquota: ChannelAccount,
) -> ClientInitiatorHandshake<T, S>where
T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
S: CoarseTimeProvider + SleepProvider,
pub fn launch<T, S>(
self,
tls: T,
sleep_prov: S,
memquota: ChannelAccount,
) -> ClientInitiatorHandshake<T, S>where
T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static,
S: CoarseTimeProvider + SleepProvider,
Launch a new client handshake over a TLS stream.
After calling this function, you’ll need to call connect() on
the result to start the handshake. If that succeeds, you’ll have
authentication info from the relay: call check() on the result
to check that. Finally, to finish the handshake, call finish()
on the result of that.