pub struct ClientInitiatorHandshake<T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider> {
sleep_prov: S,
memquota: ChannelAccount,
framed_tls: Framed<T, ChannelCellHandler>,
target_method: Option<ChannelMethod>,
unique_id: UniqId,
}Expand description
A raw client channel on which nothing has been done.
Fields
sleep_prov: SRuntime handle (insofar as we need it)
memquota: ChannelAccountMemory quota account
framed_tls: Framed<T, ChannelCellHandler>Cell encoder/decoder wrapping the underlying TLS stream
(We don’t enforce that this is actually TLS, but if it isn’t, the connection won’t be secure.)
target_method: Option<ChannelMethod>Declared target method for this channel, if any.
unique_id: UniqIdLogging identifier for this stream. (Used for logging only.)
Implementations
Sourceimpl<T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider> ClientInitiatorHandshake<T, S>
impl<T: AsyncRead + AsyncWrite + StreamOps + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider> ClientInitiatorHandshake<T, S>
Sourcepub(crate) fn new(
tls: T,
target_method: Option<ChannelMethod>,
sleep_prov: S,
memquota: ChannelAccount,
) -> Self
pub(crate) fn new( tls: T, target_method: Option<ChannelMethod>, sleep_prov: S, memquota: ChannelAccount, ) -> Self
Construct a new ClientInitiatorHandshake.
Sourcepub async fn connect<F>(
self,
now_fn: F,
) -> Result<UnverifiedClientChannel<T, S>>where
F: FnOnce() -> SystemTime,
pub async fn connect<F>(
self,
now_fn: F,
) -> Result<UnverifiedClientChannel<T, S>>where
F: FnOnce() -> SystemTime,
Negotiate a link protocol version with the relay, and read the relay’s handshake information.
Takes a function that reports the current time. In theory, this can just be
SystemTime::get().