struct PadderState<const N: usize> {
state: SmallVec<[MachineState; N]>,
}Expand description
Represents the state for all padding machines within a framework.
N should be around the number of padding machines that the framework should support.
Fields
state: SmallVec<[MachineState; N]>A list of all the padding machine states for a single framework.
This list is indexed by MachineId.
Implementations
Sourceimpl<const N: usize> PadderState<N>
impl<const N: usize> PadderState<N>
Sourcefn state_mut(&mut self, id: MachineId) -> &mut MachineState
fn state_mut(&mut self, id: MachineId) -> &mut MachineState
Sourcefn trigger_action(
&mut self,
action: &TriggerAction<Instant>,
now: Instant,
events_out: &mut SmallVec<[TriggerEvent; 1]>,
) -> bool
fn trigger_action( &mut self, action: &TriggerAction<Instant>, now: Instant, events_out: &mut SmallVec<[TriggerEvent; 1]>, ) -> bool
Execute a single TriggerAction on this state.
TriggerActions are created by maybenot::Framework instances
in response to TriggerEvents.
Executing a TriggerAction can adjust timers,
and can schedule a new ScheduledAction to be taken in the future;
it does not, however, send any padding or adjust any blocking on its own.
The current time should be provided in now.
Executing a TriggerAction can cause more events to occur.
If this happens, they are added to events_out.
If this method returns false, no timer has changed. If this method returns true, then a timer may have changed. (False positives are possible, but not false negatives.)
Sourcefn next_expiration(&self) -> Option<Instant>
fn next_expiration(&self) -> Option<Instant>
Return the next instant (if any) at which any of the padding machines’ timers will expire.