pub(super) struct ThisThreadRng;Expand description
Helper: An Rng object that calls rand::rng() to get the thread Rng.
(We use this since we want our maybenot Framework to use the thread Rng, but we can’t have it own the thread Rng. )