Timer

Struct Timer 

struct Timer {
    next_expiration: Option<Instant>,
    waker: Waker,
}
Possible state of a Framework’s aggregate timer.

(Since there are two possible timers for each Machine, we just keep track of the one that will expire next.)

Fields

next_expiration: Option<Instant>

The next time at which any of this padding machines’ timer will expire.

(None means “no timers are set.”)

waker: Waker

A Waker that we must wake whenever self.next_expiration becomes sooner than our next scheduled wakeup (as passed as an argument to set_expiration).

Implementations

Source

impl Timer

fn new() -> Self

Construct a new Timer.

fn get_expiration(&mut self, waker: &Waker) -> Option<Instant>

Return the next expiration time, and schedule waker to be alerted whenever the expiration time becomes earlier than the time at which we’ve actually decided to sleep (passed as an argument to set_expiration()).

(There are two separate expiration times at work here because, in higher-level code, we combine all the timer expirations for all padding machines on a circuit into a single expiration, and track only that expiration.)

fn set_expiration( &mut self, new_expiration: Option<Instant>, next_scheduled_wakeup: Option<Instant>, )

Change the expiration time to new_expiration, alerting the Waker if that time is earlier than next_scheduled_wakeup.