fn hopnum_from_hop_idx(hop_idx: usize) -> HopNumExpand description
Construct a HopNum from an index into the hops field of a PaddingShared.
Panics
Panics if hop_idx is greater than u8::MAX, which should be impossible.
fn hopnum_from_hop_idx(hop_idx: usize) -> HopNumConstruct a HopNum from an index into the hops field of a PaddingShared.
Panics if hop_idx is greater than u8::MAX, which should be impossible.