struct BlockingState {
hop_blocked: BitArray<[usize; 1], Lsb0>,
blocking_non_bypassable: BitArray<[usize; 1], Lsb0>,
}Expand description
Current padding-related blocking status for a circuit.
We have to keep track of whether each hop is blocked or not, and whether its blocking is bypassable. But all we actually need to tell the reactor code is whether to block the entire circuit or not.
Fields
hop_blocked: BitArray<[usize; 1], Lsb0>Whether each hop is currently blocked.
blocking_non_bypassable: BitArray<[usize; 1], Lsb0>Whether each hop’s blocking is currently not bypassable.
Implementations
Sourceimpl BlockingState
impl BlockingState
Sourcefn set_blocked(&mut self, idx: usize, is_bypassable: bool)
fn set_blocked(&mut self, idx: usize, is_bypassable: bool)
Set the hop at position idx to blocked.
Sourcefn set_unblocked(&mut self, idx: usize)
fn set_unblocked(&mut self, idx: usize)
Set the hop at position idx to unblocked.
Sourcefn blocking_update_paddingevent(&self) -> PaddingEvent
fn blocking_update_paddingevent(&self) -> PaddingEvent
Return a PaddingEvent