pub struct CircuitPadderConfig {
machines: Option<Arc<[Machine]>>,
max_outbound_blocking_frac: Option<f64>,
max_outbound_padding_frac: Option<f64>,
max_inbound_padding_frac: Option<f64>,
enforce_inbound_padding_after_cells: Option<u16>,
}Expand description
Builder for PaddingRules.
Fields
machines: Option<Arc<[Machine]>>List of padding machines to use for shaping traffic.
Note that this list may be empty, if we only want to receive padding, and never send it.
max_outbound_blocking_frac: Option<f64>Maximum allowable outbound padding fraction.
Passed directly to maybenot; not enforced in Arti.
See maybenot::Framework::new for details.
Must be between 0.0 and 1.0
max_outbound_padding_frac: Option<f64>Maximum allowable outbound blocking fraction.
Passed directly to maybenot; not enforced in Arti.
See maybenot::Framework::new for details.
Must be between 0.0 and 1.0.
max_inbound_padding_frac: Option<f64>Maximum allowable fraction of inbound padding
enforce_inbound_padding_after_cells: Option<u16>Number of cells before which we should not enforce max_inbound_padding_frac.
Implementations
Sourceimpl CircuitPadderConfig
impl CircuitPadderConfig
Sourcepub fn machines(&mut self, value: Arc<[Machine]>) -> &mut Self
pub fn machines(&mut self, value: Arc<[Machine]>) -> &mut Self
List of padding machines to use for shaping traffic.
Note that this list may be empty, if we only want to receive padding, and never send it.
Sourcepub fn max_outbound_blocking_frac(&mut self, value: f64) -> &mut Self
pub fn max_outbound_blocking_frac(&mut self, value: f64) -> &mut Self
Maximum allowable outbound padding fraction.
Passed directly to maybenot; not enforced in Arti.
See maybenot::Framework::new for details.
Must be between 0.0 and 1.0
Sourcepub fn max_outbound_padding_frac(&mut self, value: f64) -> &mut Self
pub fn max_outbound_padding_frac(&mut self, value: f64) -> &mut Self
Maximum allowable outbound blocking fraction.
Passed directly to maybenot; not enforced in Arti.
See maybenot::Framework::new for details.
Must be between 0.0 and 1.0.
Sourcepub fn max_inbound_padding_frac(&mut self, value: f64) -> &mut Self
pub fn max_inbound_padding_frac(&mut self, value: f64) -> &mut Self
Maximum allowable fraction of inbound padding
Sourcepub fn enforce_inbound_padding_after_cells(&mut self, value: u16) -> &mut Self
pub fn enforce_inbound_padding_after_cells(&mut self, value: u16) -> &mut Self
Number of cells before which we should not enforce max_inbound_padding_frac.
Sourcefn build(&self) -> Result<PaddingRules, CircuitPadderConfigError>
fn build(&self) -> Result<PaddingRules, CircuitPadderConfigError>
Sourcefn create_empty() -> Self
fn create_empty() -> Self
Create an empty builder, with all fields set to None or PhantomData.
Sourceimpl CircuitPadderConfig
impl CircuitPadderConfig
Sourcefn validate(&self) -> Result<(), CircuitPadderConfigError>
fn validate(&self) -> Result<(), CircuitPadderConfigError>
Helper: Return an error if this is not a valid Builder.
Sourcepub fn create_padder(
&self,
now: Instant,
) -> Result<CircuitPadder, CircuitPadderConfigError>
pub fn create_padder( &self, now: Instant, ) -> Result<CircuitPadder, CircuitPadderConfigError>
Construct a CircuitPadder based on this CircuitPadderConfig.
A CircuitPadderConfig is created its accessors, and used with this method to build a CircuitPadder.
That CircuitPadder can then be installed on a circuit using ClientCirc::start_padding_at_hop.