pub(crate) struct SendPadding {
machine: MachineId,
pub(crate) hop: HopNum,
pub(crate) replace: Replace,
pub(crate) bypass: Bypass,
}Expand description
An indication that we should send a padding cell.
Don’t drop this: instead, once the cell is queued,
pass this SendPadding object to the relevant PaddingController
to report that the particular piece of padding has been queued.
Fields
machine: MachineIdThe machine within a framework that told us to send the padding.
We store this so we can tell the framework which machine’s padding we sent.
hop: HopNumThe hop to which we need to send the padding.
replace: ReplaceWhether this padding can be replaced by regular data.
bypass: BypassWhether this padding cell should bypass any current blocking.
Implementations
Sourceimpl SendPadding
impl SendPadding
Sourcefn into_sent_event(self) -> TriggerEvent
fn into_sent_event(self) -> TriggerEvent
Convert this SendPadding into a TriggerEvent for Maybenot, to indicate that the padding was sent.
Sourcepub(crate) fn may_replace_with_data(&self) -> Replace
pub(crate) fn may_replace_with_data(&self) -> Replace
If true, we are allowed to replace this padding cell with a normal non-padding cell.
(If we do, we should call PaddingController::queued_data_as_padding)
Sourcepub(crate) fn may_bypass_block(&self) -> Bypass
pub(crate) fn may_bypass_block(&self) -> Bypass
Return whether this padding cell is allowed to bypass any current blocking.