pub(super) struct TunnelMutableState(Mutex<HashMap<UniqId, Arc<MutableState>>>);Expand description
The mutable state of a tunnel, shared between ClientCirc and Reactor.
NOTE(gabi): this mutex-inside-a-mutex might look suspicious,
but it is currently the best option we have for sharing
the circuit state with ClientCirc (and soon, with ClientTunnel).
In practice, these mutexes won’t be accessed very often
(they’re accessed for writing when a circuit is extended,
and for reading by the various ClientCirc APIs),
so they shouldn’t really impact performance.
Alternatively, the circuit state information could be shared
outside the reactor through a channel (passed to the reactor via a CtrlCmd),
but in #1840 @opara notes that involves making the ClientCirc accessors
(ClientCirc::path, ClientCirc::binding_key, etc.)
asynchronous, which will significantly complicate their callsites,
which would in turn need to be made async too.
We should revisit this decision at some point, and decide whether an async API would be preferable.
Tuple Fields
0: Mutex<HashMap<UniqId, Arc<MutableState>>>Implementations
Sourceimpl TunnelMutableState
impl TunnelMutableState
Sourcepub(super) fn insert(&self, unique_id: UniqId, mutable: Arc<MutableState>)
pub(super) fn insert(&self, unique_id: UniqId, mutable: Arc<MutableState>)
Add the MutableState of a circuit.
Sourcepub(super) fn remove(&self, unique_id: UniqId)
pub(super) fn remove(&self, unique_id: UniqId)
Remove the MutableState of a circuit.
Sourcefn all_paths(&self) -> Vec<Arc<Path>>
fn all_paths(&self) -> Vec<Arc<Path>>
Return a Path object describing all the circuits in this tunnel.
Sourcepub(super) fn tagged_paths(&self) -> HashMap<UniqId, Arc<Path>>
pub(super) fn tagged_paths(&self) -> HashMap<UniqId, Arc<Path>>
Return a representation of the Paths for all the circuits in this tunnel, as a map from each circuits’ UniqId to its path.
This is only exposed for the RPC subsystem, where it is documented that the
format of UniqId is not stable.
Sourcefn single_path(&self) -> Result<Arc<Path>>
fn single_path(&self) -> Result<Arc<Path>>
Return a list of Path objects describing the only circuit in this tunnel.
Returns an error if the tunnel has more than one tunnel.
Sourcefn first_hop(&self, unique_id: UniqId) -> Result<Option<OwnedChanTarget>>
fn first_hop(&self, unique_id: UniqId) -> Result<Option<OwnedChanTarget>>
Return a description of the first hop of this circuit.
Returns an error if a circuit with the specified UniqId doesn’t exist.
Returns Ok(None) if the specified circuit doesn’t have any hops.