fn resolvedval_to_result(val: ResolvedVal) -> Result<ResolvedVal>Expand description
Convert a ResolvedVal into a Result, based on whether or not
it represents an error.
fn resolvedval_to_result(val: ResolvedVal) -> Result<ResolvedVal>Convert a ResolvedVal into a Result, based on whether or not
it represents an error.