pub(super) enum CircuitCmd {
Send(SendRelayCell),
HandleSendMe {
hop: HopNum,
sendme: Sendme,
},
CloseStream {
hop: HopNum,
sid: StreamId,
behav: CloseStreamBehavior,
reason: TerminateReason,
},
Conflux(ConfluxCmd),
CleanShutdown,
Enqueue(OooRelayMsg),
}Expand description
A command to run in response to a circuit event.
Unlike RunOnceCmdInner, doesn’t know anything about UniqIds.
The user of the CircuitCmds is supposed to know the UniqId
of the circuit the CircuitCmd came from.
This type gets mapped to a RunOnceCmdInner in the circuit reactor.
Variants
Send(SendRelayCell)
Send a RELAY cell on the circuit leg this command originates from.
HandleSendMe
Handle a SENDME message received on the circuit leg this command originates from.
CloseStream
Close the specified stream on the circuit leg this command originates from.
Fields

behav: CloseStreamBehaviorThe stream-closing behavior.

reason: TerminateReasonThe reason for closing the stream.
Conflux(ConfluxCmd)
Perform an action resulting from handling a conflux cell.
CleanShutdown
Perform a clean shutdown on this circuit.
Enqueue(OooRelayMsg)
Enqueue an out-of-order cell in the reactor.