pub(crate) struct ClientConfluxMsgHandler {
state: ConfluxState,
nonce: V1Nonce,
join_point: HopNum,
init_rtt: Option<Duration>,
link_sent: Option<SystemTime>,
runtime: DynTimeProvider,
last_seq_recv: u64,
last_seq_sent: u64,
last_seq_delivered: Arc<AtomicU64>,
have_seen_switch: bool,
cells_since_switch: usize,
cwnd_params: CongestionWindowParams,
}Expand description
Client-side implementation of a conflux message handler.
Fields
state: ConfluxStateThe current state this leg is in.
nonce: V1NonceThe nonce associated with the circuits from this set.
join_point: HopNumThe expected conflux join point.
init_rtt: Option<Duration>measured during the conflux handshake.
On the client side, this is the RTT between
RELAY_CONFLUX_LINK and RELAY_CONFLUX_LINKED.
link_sent: Option<SystemTime>The time when the handshake was initiated.
runtime: DynTimeProviderA handle to the time provider.
last_seq_recv: u64The sequence number of the last message received on this leg.
This is a relative number.
Incremented by the ConfluxMsgHandler
each time a cell that counts towards sequence numbers is received on this leg.
last_seq_sent: u64The sequence number of the last message sent on this leg.
This is a relative number.
Incremented by the ConfluxMsgHandler
each time a cell that counts towards sequence numbers is sent on this leg.
last_seq_delivered: Arc<AtomicU64>The absolute sequence number of the last message delivered to a stream.
This is shared by all the circuits in a conflux set,
Incremented by the ConfluxMsgHandler
upon delivering the cell to its corresponding stream.
have_seen_switch: boolWhether we have processed any SWITCH cells on the leg this handler is installed on.
cells_since_switch: usizeThe number of cells that count towards the conflux seqnos received on this leg since the last SWITCH.
cwnd_params: CongestionWindowParamsThe congestion window parameters.
Used for SWITCH cell validation.
Implementations
Sourceimpl ClientConfluxMsgHandler
impl ClientConfluxMsgHandler
Sourcepub(crate) fn new(
join_point: HopNum,
nonce: V1Nonce,
last_seq_delivered: Arc<AtomicU64>,
cwnd_params: CongestionWindowParams,
runtime: DynTimeProvider,
) -> Self
pub(crate) fn new( join_point: HopNum, nonce: V1Nonce, last_seq_delivered: Arc<AtomicU64>, cwnd_params: CongestionWindowParams, runtime: DynTimeProvider, ) -> Self
Create a new client conflux message handler.
Sourcefn handle_conflux_link(
&mut self,
msg: UnparsedRelayMsg,
hop: HopNum,
) -> Result<Option<ConfluxCmd>>
fn handle_conflux_link( &mut self, msg: UnparsedRelayMsg, hop: HopNum, ) -> Result<Option<ConfluxCmd>>
Handle a conflux LINK request coming from the specified hop.
Sourcefn handle_conflux_linked(
&mut self,
msg: UnparsedRelayMsg,
hop: HopNum,
) -> Result<Option<ConfluxCmd>>
fn handle_conflux_linked( &mut self, msg: UnparsedRelayMsg, hop: HopNum, ) -> Result<Option<ConfluxCmd>>
Handle a conflux LINKED response coming from the specified hop.
The caller must validate the hop the cell originated from.
To prevent DROPMARK attacks, this returns a protocol error if any of the following conditions are not met:
- this is an unlinked circuit that sent a LINK command
- that the nonce matches the nonce used in the LINK command
Sourcefn handle_conflux_linked_ack(
&mut self,
msg: UnparsedRelayMsg,
hop: HopNum,
) -> Result<Option<ConfluxCmd>>
fn handle_conflux_linked_ack( &mut self, msg: UnparsedRelayMsg, hop: HopNum, ) -> Result<Option<ConfluxCmd>>
Handle a conflux LINKED acknowledgment coming from the specified hop.
Sourcefn handle_conflux_switch(
&mut self,
msg: UnparsedRelayMsg,
_hop: HopNum,
) -> Result<Option<ConfluxCmd>>
fn handle_conflux_switch( &mut self, msg: UnparsedRelayMsg, _hop: HopNum, ) -> Result<Option<ConfluxCmd>>
Handle a conflux SWITCH coming from the specified hop.
Sourcefn validate_switch_seqno(&self, rel_seqno: u32) -> Result<()>
fn validate_switch_seqno(&self, rel_seqno: u32) -> Result<()>
Validate the relative sequence number specified in a switch command.
Returns an error if
rel_seqnois 0 (i.e. the SWITCH cell does not actually increment thelast_seq_recvseqno on this leg)- the tunnel has not yet received any data and
rel_seqnois greater than the initial congestion window,