pub(crate) enum CtrlCmd {
Shutdown,
ExtendVirtual {
cell_crypto: (Box<dyn OutboundClientLayer + Send>, Box<dyn InboundClientLayer + Send>, Option<CircuitBinding>),
settings: HopSettings,
done: Sender<Result<()>>,
},
ResolveTargetHop {
hop: TargetHop,
done: Sender<Result<HopLocation>>,
},
AwaitStreamRequest {
incoming_sender: Sender<StreamReqInfo, MpscSpec>,
cmd_checker: Box<dyn CmdChecker + Send + 'static>,
done: Sender<Result<()>>,
hop: TargetHop,
filter: Box<dyn IncomingStreamRequestFilter>,
},
GetBindingKey {
hop: TargetHop,
done: Sender<Result<Option<CircuitBinding>>>,
},
ShutdownAndReturnCircuit {
answer: Sender<Result<Circuit, Bug>>,
},
SetPadder {
hop: HopLocation,
padder: Option<CircuitPadder>,
sender: Sender<Result<()>>,
},
GetTunnelActivity {
sender: Sender<TunnelActivity>,
},
}Expand description
A message telling the reactor to do something.
The difference between this and CtrlMsg is that CtrlCmds
never cause cells to sent on the channel,
while CtrlMsgs potentially do: CtrlMsgs are mapped to RunOnceCmdInner commands,
some of which instruct the reactor to send cells down the channel.
Variants
Shutdown
Shut down the reactor.
ExtendVirtual
Extend the circuit by one hop, in response to an out-of-band handshake.
(This is used for onion services, where the negotiation takes place in INTRODUCE and RENDEZVOUS messages.)
Fields
cell_crypto: (Box<dyn OutboundClientLayer + Send>, Box<dyn InboundClientLayer + Send>, Option<CircuitBinding>)The cryptographic algorithms and keys to use when communicating with the newly added hop.
settings: HopSettingsA set of parameters to negotiate with this hop.
ResolveTargetHop
Resolve a given TargetHop into a precise HopLocation.
Fields
done: Sender<Result<HopLocation>>Oneshot channel to notify on completion.
AwaitStreamRequest
Begin accepting streams on this circuit.
Fields
incoming_sender: Sender<StreamReqInfo, MpscSpec>A channel for sending information about an incoming stream request.
cmd_checker: Box<dyn CmdChecker + Send + 'static>A CmdChecker to keep track of which message types are acceptable.
filter: Box<dyn IncomingStreamRequestFilter>A filter used to check requests before passing them on.
GetBindingKey
Request the binding key of a target hop.
Fields
done: Sender<Result<Option<CircuitBinding>>>Oneshot channel to notify on completion.
ShutdownAndReturnCircuit
Shut down the reactor, and return the underlying Circuit,
if the tunnel is not multi-path.
Returns an error if called on a multi-path reactor.
Fields
SetPadder
Install or remove a padding::CircuitPadder for a given hop.
Any existing CircuitPadder at that hop is replaced.
Fields
hop: HopLocationThe hop to modify.
padder: Option<CircuitPadder>The Padder to install, or None to remove any existing padder.
GetTunnelActivity
Yield the most active TunnelActivity for any hop on any leg of this tunnel.
Fields
sender: Sender<TunnelActivity>A sender to receive the reply.