send_conflux_outcome

Function send_conflux_outcome 

fn send_conflux_outcome(
    tx: Sender<Result<Vec<Result<(), ConfluxHandshakeError>>>>,
    res: Result<Vec<Result<(), ConfluxHandshakeError>>>,
) -> Result<(), ReactorError>
Notify the conflux handshake initiator of the handshake outcome.

Returns an error if the initiator has done away.