ConfluxLinkResultChannel

Type Alias ConfluxLinkResultChannel 

pub(super) type ConfluxLinkResultChannel = Sender<Result<Vec<Result<(), ConfluxHandshakeError>>>>;
The type of oneshot channel used to inform reactor users of the outcome of a client-side conflux handshake.

Contains a list of handshake results, one for each circuit that we were asked to link in the tunnel.

Aliased Type

pub(super) struct ConfluxLinkResultChannel { /* private fields */ }