pub struct ClientDataStreamCtrl {
tunnel: Weak<ClientTunnel>,
status: Arc<Mutex<DataStreamStatus>>,
_memquota: StreamAccount,
}Expand description
An object used to control and monitor a data stream.
Notes
This is a separate type from DataStream because it’s useful to have
multiple references to this object, whereas a DataReader and DataWriter
need to have a single owner for the AsyncRead and AsyncWrite APIs to
work correctly.
Fields
tunnel: Weak<ClientTunnel>The circuit to which this stream is attached.
Note that the stream’s reader and writer halves each contain a StreamTarget,
which in turn has a strong reference to the ClientCirc. So as long as any
one of those is alive, this reference will be present.
We make this a Weak reference so that once the stream itself is closed, we can’t leak circuits.
status: Arc<Mutex<DataStreamStatus>>Shared user-visible information about the state of this stream.
TODO RPC: This will probably want to be a postage::Watch or something
similar, if and when it stops moving around.
_memquota: StreamAccountThe memory quota account that should be used for this stream’s data
Exists to keep the account alive
Implementations
Sourceimpl ClientDataStreamCtrl
impl ClientDataStreamCtrl
Sourcepub fn is_connected(&self) -> bool
pub fn is_connected(&self) -> bool
Return true if the underlying stream is connected. (That is, if it has
received a CONNECTED message, and has not been closed.)