DataReaderInner

Struct DataReaderInner 

pub(crate) struct DataReaderInner {
    state: Option<DataReaderState>,
    _memquota: StreamAccount,
    ctrl: Arc<ClientDataStreamCtrl>,
}
The inner reader for DataReader.

This type is responsible for taking stream messages and extracting the stream data from them. Flow control logic is implemented in DataReader to avoid making this type more complex.

Fields

state: Option<DataReaderState>

Internal state for this reader.

This is stored in an Option so that we can mutate it in poll_read(). It might be possible to do better here, and we should refactor if so.

_memquota: StreamAccount

The memory quota account that should be used for this stream’s data

Exists to keep the account alive

ctrl: Arc<ClientDataStreamCtrl>

A control object that can be used to monitor and control this stream without needing to own it.

Implementations

Source

impl DataReaderInner

pub(crate) fn client_stream_ctrl(&self) -> &Arc<ClientDataStreamCtrl>

Return a ClientDataStreamCtrl object that can be used to monitor and interact with this stream without holding the stream itself.