struct DataStreamStatus {
received_connected: bool,
sent_end: bool,
received_end: bool,
received_err: bool,
}Expand description
Shared status flags for tracking the status of as DataStream.
We expect to refactor this a bit, so it’s not exposed at all.
Fields
received_connected: boolTrue if we’ve received a CONNECTED message.
sent_end: boolTrue if we have decided to send an END message.
received_end: boolTrue if we have received an END message telling us to close the stream.
received_err: boolTrue if we have received an error.
(This is not a subset or superset of received_end; some errors are END messages but some aren’t; some END messages are errors but some aren’t.)
Implementations
Sourceimpl DataStreamStatus
impl DataStreamStatus
Sourcefn record_connected(&mut self)
fn record_connected(&mut self)
Remember that we’ve received a connected message.
Sourcefn record_error(&mut self, e: &Error)
fn record_error(&mut self, e: &Error)
Remember that we’ve received an error of some kind.