DataWriterInner

Struct DataWriterInner 

struct DataWriterInner {
    state: Option<DataWriterState>,
    _memquota: StreamAccount,
    ctrl: Arc<ClientDataStreamCtrl>,
}
The inner writer for DataWriter.

This type is responsible for taking bytes and packaging them into cells. Rate limiting is implemented in DataWriter to avoid making this type more complex.

Fields

state: Option<DataWriterState>

Internal state for this writer

This is stored in an Option so that we can mutate it in the AsyncWrite functions. It might be possible to do better here, and we should refactor if so.

_memquota: StreamAccount

The memory quota account that should be used for this stream’s data

Exists to keep the account alive

ctrl: Arc<ClientDataStreamCtrl>

A control object that can be used to monitor and control this stream without needing to own it.

Implementations

Source

impl DataWriterInner

fn client_stream_ctrl(&self) -> &Arc<ClientDataStreamCtrl>

fn poll_flush_impl( self: Pin<&mut Self>, cx: &mut Context<'_>, should_close: bool, ) -> Poll<IoResult<()>>

Helper for poll_flush() and poll_close(): Performs a flush, then closes the stream if should_close is true.