ConfluxMsgHandler

Struct ConfluxMsgHandler 

pub(crate) struct ConfluxMsgHandler {
    handler: Box<dyn AbstractConfluxMsgHandler + Send + Sync>,
    last_seq_delivered: Arc<AtomicU64>,
}
Cell handler for conflux cells.

One per Circuit.

Fields

handler: Box<dyn AbstractConfluxMsgHandler + Send + Sync>

Inner message handler

Customizes the cell handling logic, because clients and exits behave differently.

TODO: can/should we avoid dynamic dispatch here?

last_seq_delivered: Arc<AtomicU64>

The absolute sequence number of the last message delivered to a stream.

This is shared by all the circuits in a conflux set.

Implementations

Source

impl ConfluxMsgHandler

pub(crate) fn new( handler: Box<dyn AbstractConfluxMsgHandler + Send + Sync>, last_seq_delivered: Arc<AtomicU64>, ) -> Self

Create a new message handler using the specified AbstractConfluxMsgHandler.

Clients and relays both use this function.

fn validate_source_hop(&self, msg: &UnparsedRelayMsg, hop: HopNum) -> Result<()>

Validate the specified source hop of a conflux cell.

The client impl of this function returns an error if the hop is not the last hop.

The exit impl of this function returns an error if the source hop is not the last hop, or if there are further hops after the source hop.

pub(crate) fn handle_conflux_msg( &mut self, msg: UnparsedRelayMsg, hop: HopNum, ) -> Option<ConfluxCmd>

Handle the specified conflux msg.

Client-only: note that the LINK cell was sent.

Used for the initial RTT measurement.

pub(crate) fn handshake_timeout(&self) -> Option<SystemTime>

Get the wallclock time when the handshake on this circuit is supposed to time out.

Returns None if the handshake is not currently in progress.

pub(crate) fn status(&self) -> ConfluxStatus

Returns the conflux status of this handler.

fn is_msg_in_order(&self) -> Result<bool, Bug>

Check our sequence numbers to see if the current msg is in order.

Returns an internal error if the relative seqno is lower than the absolute seqno.

fn prepare_ooo_entry( &self, hopnum: HopNum, cell_counts_towards_windows: bool, streamid: StreamId, msg: UnparsedRelayMsg, ) -> OooRelayMsg

Return a OooRelayMsg for the reactor to buffer.

pub(crate) fn action_for_msg( &mut self, hopnum: HopNum, cell_counts_towards_windows: bool, streamid: StreamId, msg: UnparsedRelayMsg, ) -> Result<ConfluxAction, Bug>

Check the sequence number of the specified msg, and decide whether it should be delivered or buffered.

pub(crate) fn inc_last_seq_delivered(&self, msg: &UnparsedRelayMsg)

Increment the absolute “delivered” seqno for this conflux set if the specified message counts towards sequence numbers.

pub(crate) fn init_rtt(&self) -> Option<Duration>

Returns the initial RTT measured by this handler.

pub(crate) fn last_seq_sent(&self) -> u64

Return the sequence number of the last message sent on this leg.

pub(crate) fn set_last_seq_sent(&mut self, n: u64)

Set the sequence number of the last message sent on this leg.

pub(crate) fn last_seq_recv(&self) -> u64

Return the sequence number of the last message received on this leg.

pub(crate) fn note_cell_sent(&mut self, cmd: RelayCmd)

Note that a cell has been sent.

Updates the internal sequence numbers.