pub(crate) struct FixedWindow {
recvwindow: RecvWindow<CircParams>,
sendwindow: SendWindow<CircParams>,
params: FixedWindowParams,
}Expand description
Fixed window algorithm which is essentially the SENDME v0 with fixed receive and send window size.
Fields
recvwindow: RecvWindow<CircParams>Window used to say how many cells we can receive.
sendwindow: SendWindow<CircParams>Window used to say how many cells we can send.
params: FixedWindowParamsThe params from the consensus.
Implementations
Sourceimpl FixedWindow
impl FixedWindow
Sourcepub(crate) fn new(params: FixedWindowParams) -> Self
pub(crate) fn new(params: FixedWindowParams) -> Self
Create a new FixedWindow form a given initial sendwindow size.
Note: the initial recvwindow size is given by sendme::CircParams::start.