is_sendme_inc_valid

Function is_sendme_inc_valid 

pub(crate) fn is_sendme_inc_valid(
    inc: u8,
    params: &CongestionControlParams,
) -> bool
Return true iff the given sendme increment is valid with regards to the value in the circuit parameters that is taken from the consensus.