pub struct CongestionControlParamsBuilder {
alg: Option<Algorithm>,
fixed_window_params: Option<FixedWindowParams>,
cwnd_params: Option<CongestionWindowParams>,
rtt_params: Option<RoundTripEstimatorParams>,
}Expand description
Builder for CongestionControlParams.
Fields
alg: Option<Algorithm>The congestion control algorithm to use.
fixed_window_params: Option<FixedWindowParams>Parameters to the fallback fixed-window algorithm, which we use
when the one in alg is not supported by a given relay.
It is put in here because by the time we do path selection, we don’t have access to the consensus and so we have to keep our fallback ready.
cwnd_params: Option<CongestionWindowParams>Congestion window parameters.
rtt_params: Option<RoundTripEstimatorParams>RTT calculation parameters.
Implementations
Sourceimpl CongestionControlParamsBuilder
impl CongestionControlParamsBuilder
Sourcepub fn fixed_window_params(&mut self, value: FixedWindowParams) -> &mut Self
pub fn fixed_window_params(&mut self, value: FixedWindowParams) -> &mut Self
Parameters to the fallback fixed-window algorithm, which we use
when the one in alg is not supported by a given relay.
It is put in here because by the time we do path selection, we don’t have access to the consensus and so we have to keep our fallback ready.
Sourcepub fn cwnd_params(&mut self, value: CongestionWindowParams) -> &mut Self
pub fn cwnd_params(&mut self, value: CongestionWindowParams) -> &mut Self
Congestion window parameters.
Sourcepub fn rtt_params(&mut self, value: RoundTripEstimatorParams) -> &mut Self
pub fn rtt_params(&mut self, value: RoundTripEstimatorParams) -> &mut Self
RTT calculation parameters.
Sourcepub fn build(&self) -> Result<CongestionControlParams, ConfigBuildError>
pub fn build(&self) -> Result<CongestionControlParams, ConfigBuildError>
Sourcefn create_empty() -> Self
fn create_empty() -> Self
Create an empty builder, with all fields set to None or PhantomData.