pub struct CongestionWindowParamsBuilder {
cwnd_init: Option<u32>,
cwnd_inc_pct_ss: Option<Percentage<u32>>,
cwnd_inc: Option<u32>,
cwnd_inc_rate: Option<u32>,
cwnd_min: Option<u32>,
cwnd_max: Option<u32>,
sendme_inc: Option<u32>,
}Expand description
Builder for CongestionWindowParams.
Fields
cwnd_init: Option<u32>Initial size of the congestion window.
cwnd_inc_pct_ss: Option<Percentage<u32>>Percent of cwnd to increment by during slow start.
cwnd_inc: Option<u32>Number of cells to increment cwnd by during steady state.
cwnd_inc_rate: Option<u32>Number of times per congestion window to update based on congestion signals.
cwnd_min: Option<u32>Minimum congestion window (must be at least sendme_inc)
cwnd_max: Option<u32>Maximum congestion window
sendme_inc: Option<u32>The SENDME increment as in the number of cells to ACK with every SENDME. This is coming from the consensus and negotiated during circuit setup.
Implementations
Sourceimpl CongestionWindowParamsBuilder
impl CongestionWindowParamsBuilder
Sourcepub fn cwnd_inc_pct_ss(&mut self, value: Percentage<u32>) -> &mut Self
pub fn cwnd_inc_pct_ss(&mut self, value: Percentage<u32>) -> &mut Self
Percent of cwnd to increment by during slow start.
Sourcepub fn cwnd_inc(&mut self, value: u32) -> &mut Self
pub fn cwnd_inc(&mut self, value: u32) -> &mut Self
Number of cells to increment cwnd by during steady state.
Sourcepub fn cwnd_inc_rate(&mut self, value: u32) -> &mut Self
pub fn cwnd_inc_rate(&mut self, value: u32) -> &mut Self
Number of times per congestion window to update based on congestion signals.
Sourcepub fn cwnd_min(&mut self, value: u32) -> &mut Self
pub fn cwnd_min(&mut self, value: u32) -> &mut Self
Minimum congestion window (must be at least sendme_inc)
Sourcepub fn sendme_inc(&mut self, value: u32) -> &mut Self
pub fn sendme_inc(&mut self, value: u32) -> &mut Self
The SENDME increment as in the number of cells to ACK with every SENDME. This is coming from the consensus and negotiated during circuit setup.
Sourcepub fn build(&self) -> Result<CongestionWindowParams, ConfigBuildError>
pub fn build(&self) -> Result<CongestionWindowParams, ConfigBuildError>
Sourcefn create_empty() -> Self
fn create_empty() -> Self
Create an empty builder, with all fields set to None or PhantomData.