pub(crate) fn cell_counts_towards_windows(cell: &UnparsedRelayMsg) -> boolExpand description
Return true if this message is counted by flow-control windows.
pub(crate) fn cell_counts_towards_windows(cell: &UnparsedRelayMsg) -> boolReturn true if this message is counted by flow-control windows.