pub(crate) struct CongestionControl {
state: State,
sendme_validator: SendmeValidator<SendmeTag>,
rtt: RoundtripTimeEstimator,
algorithm: Box<dyn CongestionControlAlgorithm>,
}Expand description
Congestion control state of a hop on a circuit.
This controls the entire logic of congestion control and circuit level SENDMEs.
Fields
state: StateWhich congestion control state are we in?
sendme_validator: SendmeValidator<SendmeTag>This is the SENDME validator as in it keeps track of the circuit tag found within an authenticated SENDME cell. It can store the tags and validate a tag against our queue of expected values.
rtt: RoundtripTimeEstimatorThe RTT estimator for the circuit we are attached on.
algorithm: Box<dyn CongestionControlAlgorithm>The congestion control algorithm.
Implementations
Sourceimpl CongestionControl
impl CongestionControl
Sourcepub(crate) fn new(params: &CongestionControlParams) -> Self
pub(crate) fn new(params: &CongestionControlParams) -> Self
Construct a new CongestionControl
Sourcepub(crate) fn uses_stream_sendme(&self) -> bool
pub(crate) fn uses_stream_sendme(&self) -> bool
Return true iff the underlying algorithm uses stream level SENDMEs. At the moment, only FixedWindow uses it. It has been eliminated with Vegas.
Sourcepub(crate) fn uses_xon_xoff(&self) -> bool
pub(crate) fn uses_xon_xoff(&self) -> bool
Return true iff the underlying algorithm uses stream level XON/XOFFs. At the moment, only Vegas uses it.
Sourcepub(crate) fn can_send(&self) -> bool
pub(crate) fn can_send(&self) -> bool
Return true iff a DATA cell is allowed to be sent based on the congestion control state.
Sourcepub(crate) fn note_sendme_received(
&mut self,
runtime: &DynTimeProvider,
tag: SendmeTag,
signals: CongestionSignals,
) -> Result<()>
pub(crate) fn note_sendme_received( &mut self, runtime: &DynTimeProvider, tag: SendmeTag, signals: CongestionSignals, ) -> Result<()>
Called when a SENDME cell is received.
An error is returned if there is a protocol violation with regards to congestion control.
Sourcepub(crate) fn note_sendme_sent(&mut self) -> Result<()>
pub(crate) fn note_sendme_sent(&mut self) -> Result<()>
Called when a SENDME cell is sent.
Sourcepub(crate) fn note_data_received(&mut self) -> Result<bool>
pub(crate) fn note_data_received(&mut self) -> Result<bool>
Called when a DATA cell is received.
Returns true iff a SENDME should be sent false otherwise. An error is returned if there is a protocol violation with regards to flow or congestion control.
Sourcepub(crate) fn note_data_sent<U>(
&mut self,
runtime: &DynTimeProvider,
tag: &U,
) -> Result<()>
pub(crate) fn note_data_sent<U>( &mut self, runtime: &DynTimeProvider, tag: &U, ) -> Result<()>
Called when a DATA cell is sent.
An error is returned if there is a protocol violation with regards to flow or congestion control.
Sourcepub(crate) fn inflight(&self) -> Option<u32>
pub(crate) fn inflight(&self) -> Option<u32>
Return the number of in-flight cells (sent but awaiting SENDME ack).
Optional, because not all algorithms track this.
Sourcepub(crate) fn cwnd(&self) -> Option<CongestionWindow>
pub(crate) fn cwnd(&self) -> Option<CongestionWindow>
Return the congestion window object.
Optional, because not all algorithms track this.
Sourcepub(crate) fn rtt(&self) -> &RoundtripTimeEstimator
pub(crate) fn rtt(&self) -> &RoundtripTimeEstimator
Return a reference to the RTT estimator.