pub(crate) struct Vegas {
params: VegasParams,
bdp: BdpEstimator,
cwnd: CongestionWindow,
num_cell_until_sendme: u32,
num_sendme_until_cwnd_update: u32,
num_sendme_per_cwnd: u32,
num_inflight: u32,
is_blocked_on_chan: bool,
}Expand description
Congestion control Vegas algorithm.
TCP Vegas control algorithm estimates the queue lengths at relays by subtracting the current BDP estimate from the current congestion window.
This object implements CongestionControlAlgorithm trait used by the [‘CongestionControl’].
Spec: prop324 section 3.3 (TOR_VEGAS) C-tor: Split between congestion_control_vegas.c and the congestion_control_t struct.
Fields
params: VegasParamsCongestion control parameters.
bdp: BdpEstimatorBandwidth delay product. C-tor: “bdp”
cwnd: CongestionWindowCongestion window. C-tor: “cwnd”, “cwnd_inc_pct_ss”, “cwnd_inc”, “cwnd_min”, “cwnd_inc_rate”, “cwnd_full”,
num_cell_until_sendme: u32Number of cells expected before we send a SENDME resulting in more data.
num_sendme_until_cwnd_update: u32The number of SENDME until we will acknowledge a congestion event again. C-tor: “next_cc_event”
num_sendme_per_cwnd: u32Counts down until we process a cwnd worth of SENDME acks. Used to track full cwnd status. C-tor: “next_cwnd_event”
num_inflight: u32Number of cells in-flight (sent but awaiting SENDME ack). C-tor: “inflight”
is_blocked_on_chan: boolIndicate if we noticed we were blocked on channel during an algorithm run. This is used to notice a change from blocked to non-blocked in order to reset the num_sendme_per_cwnd. C-tor: “blocked_chan”
Implementations
Sourceimpl Vegas
impl Vegas
Sourcepub(crate) fn new(
params: VegasParams,
state: &State,
cwnd: CongestionWindow,
) -> Self
pub(crate) fn new( params: VegasParams, state: &State, cwnd: CongestionWindow, ) -> Self
Create a new Vegas from the specified parameters, state, and cwnd.