BlkCipher

Trait BlkCipher 

pub trait BlkCipher:
    BlockCipher
    + KeyInit
    + BlockSizeUser<BlockSize = U16>
    + Clone {
    const KEY_LEN: usize;
}
Helper trait to define the features we need from a block cipher, and make our “where” declarations smaller.

Not sealed because it is never used outside of this crate.

Required Associated Constants

const KEY_LEN: usize

Length of the key used by this block cipher.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementations on Foreign Types

Source

impl BlkCipher for Aes128

Source

impl BlkCipher for Aes128Dec

Source

impl BlkCipher for Aes128Enc

Source

impl BlkCipher for Aes256

Source

impl BlkCipher for Aes256Dec

Source

impl BlkCipher for Aes256Enc

Implementors