pub struct RelayCellBody(BoxedCellBody);Expand description
Type for the body of a relay cell.
Tuple Fields
0: BoxedCellBodyImplementations
Sourceimpl RelayCellBody
Functions on RelayCellBody that implement the digest/recognized
algorithm.
impl RelayCellBody
Functions on RelayCellBody that implement the digest/recognized algorithm.
The current relay crypto protocol uses two wholly inadequate fields to see whether a cell is intended for its current recipient: a two-byte “recognized” field that needs to be all-zero; and a four-byte “digest” field containing a running digest of all cells (for this recipient) to this one, seeded with an initial value (either Df or Db in the spec).
These operations are described in tor-spec section 6.1 “Relay cells”
Sourcefn recognized(&self) -> &[u8] ⓘ
fn recognized(&self) -> &[u8] ⓘ
Returns the byte slice of the recognized field.
Sourcefn recognized_mut(&mut self) -> &mut [u8] ⓘ
fn recognized_mut(&mut self) -> &mut [u8] ⓘ
Returns the mut byte slice of the recognized field.
Sourcefn digest_mut(&mut self) -> &mut [u8] ⓘ
fn digest_mut(&mut self) -> &mut [u8] ⓘ
Returns the mut byte slice of the digest field.
Sourcepub fn set_digest<D: Digest + Clone>(
&mut self,
d: &mut D,
sendme_tag: &mut SendmeTag,
)
pub fn set_digest<D: Digest + Clone>( &mut self, d: &mut D, sendme_tag: &mut SendmeTag, )
Prepare a cell body by setting its digest and recognized field.
Sourcepub fn is_recognized<D: Digest + Clone>(
&self,
d: &mut D,
rcvd: &mut SendmeTag,
) -> bool
pub fn is_recognized<D: Digest + Clone>( &self, d: &mut D, rcvd: &mut SendmeTag, ) -> bool
Check whether this just-decrypted cell is now an authenticated plaintext.
This method returns true if the recognized field is all zeros, and if the
digest field is a digest of the correct material.
If it returns true, it also sets rcvg to the appropriate authenticated
SENDME tag to use if acknowledging this message.
If this method returns false, then either further decryption is required, or the cell is corrupt.