CryptState

Struct CryptState 

struct CryptState<SC: StreamCipher, D: Digest + Clone> {
    cipher: SC,
    digest: D,
    last_sendme_tag: SendmeTag,
}
A CryptState represents one layer of shared cryptographic state between a relay and a client for a single hop, in a single direction.

For example, if a client makes a 3-hop circuit, then it will have 6 CryptStates, one for each relay, for each direction of communication.

Note that although CryptState is used to implement OutboundClientLayer, InboundClientLayer, OutboundRelayLayer, and InboundRelayLayer, each instance will only be used for one of these roles.

It is parameterized on a stream cipher and a digest type: most circuits will use AES-128-CTR and SHA1, but v3 onion services use AES-256-CTR and SHA-3.

Fields

cipher: SC

Stream cipher for en/decrypting cell bodies.

This cipher is the one keyed with Kf or Kb in the spec.

digest: D

Digest for authenticating cells to/from this hop.

This digest is the one keyed with Df or Db in the spec.

last_sendme_tag: SendmeTag

Most recent digest value generated by this crypto.