pub trait ClientLayer<F, B>where
F: OutboundClientLayer,
B: InboundClientLayer,{
// Required method
fn split_client_layer(self) -> (F, B, CircuitBinding);
}Expand description
A paired object containing the inbound and outbound cryptographic layers used by a client to communicate with a single hop on one of its circuits.
TODO: Maybe we should fold this into CryptInit.
Required Methods
Sourcefn split_client_layer(self) -> (F, B, CircuitBinding)
fn split_client_layer(self) -> (F, B, CircuitBinding)
Consume this ClientLayer and return a paired forward and reverse
crypto layer, and a CircuitBinding object