pub trait InboundRelayLayer {
// Required methods
fn originate(&mut self, cmd: ChanCmd, cell: &mut RelayCellBody) -> SendmeTag;
fn encrypt_inbound(&mut self, cmd: ChanCmd, cell: &mut RelayCellBody);
}Expand description
Represents a relay’s view of the inbound crypto state on a given circuit.
Required Methods
Sourcefn originate(&mut self, cmd: ChanCmd, cell: &mut RelayCellBody) -> SendmeTag
fn originate(&mut self, cmd: ChanCmd, cell: &mut RelayCellBody) -> SendmeTag
Prepare a RelayCellBody to be sent towards the client, and encrypt it.
Return the authentication tag.
Sourcefn encrypt_inbound(&mut self, cmd: ChanCmd, cell: &mut RelayCellBody)
fn encrypt_inbound(&mut self, cmd: ChanCmd, cell: &mut RelayCellBody)
Encrypt a RelayCellBody that is moving towards the client.