pub trait OutboundClientLayer {
// Required methods
fn originate_for(
&mut self,
cmd: ChanCmd,
cell: &mut RelayCellBody,
) -> SendmeTag;
fn encrypt_outbound(&mut self, cmd: ChanCmd, cell: &mut RelayCellBody);
}Expand description
A client’s view of the cryptographic state shared with a single relay on a circuit, as used for outbound cells.
Required Methods
Sourcefn originate_for(&mut self, cmd: ChanCmd, cell: &mut RelayCellBody) -> SendmeTag
fn originate_for(&mut self, cmd: ChanCmd, cell: &mut RelayCellBody) -> SendmeTag
Prepare a RelayCellBody to be sent to the relay at this layer, and encrypt it.
Return the authentication tag.
Sourcefn encrypt_outbound(&mut self, cmd: ChanCmd, cell: &mut RelayCellBody)
fn encrypt_outbound(&mut self, cmd: ChanCmd, cell: &mut RelayCellBody)
Encrypt a RelayCellBody to be decrypted by this layer.