pub trait RelayLayer<F, B>where
F: OutboundRelayLayer,
B: InboundRelayLayer,{
// Required method
fn split_relay_layer(self) -> (F, B, CircuitBinding);
}Expand description
A paired object containing the inbound and outbound cryptographic layers used by a relay to implement a client’s circuits.
Required Methods
Sourcefn split_relay_layer(self) -> (F, B, CircuitBinding)
fn split_relay_layer(self) -> (F, B, CircuitBinding)
Consume this ClientLayer and return a paired forward and reverse
crypto layers, and a CircuitBinding object