fn client_handshake2_ntor_v1<T>(
msg: T,
state: &NtorHandshakeState,
) -> Result<NtorHkdfKeyGenerator>Expand description
Complete a client handshake, returning a key generator on success.
fn client_handshake2_ntor_v1<T>(
msg: T,
state: &NtorHandshakeState,
) -> Result<NtorHkdfKeyGenerator>Complete a client handshake, returning a key generator on success.