fn client_handshake_ntor_v1<R>(
rng: &mut R,
relay_public: &NtorPublicKey,
) -> Result<(NtorHandshakeState, Vec<u8>)>Expand description
Perform a client handshake, generating an onionskin and a state object
fn client_handshake_ntor_v1<R>(
rng: &mut R,
relay_public: &NtorPublicKey,
) -> Result<(NtorHandshakeState, Vec<u8>)>Perform a client handshake, generating an onionskin and a state object