client_handshake_ntor_v3

Function client_handshake_ntor_v3 

fn client_handshake_ntor_v3<R: RngCore + CryptoRng>(
    rng: &mut R,
    relay_public: &NtorV3PublicKey,
    client_msg: &[u8],
    verification: &[u8],
) -> EncodeResult<(NtorV3HandshakeState, Vec<u8>)>
Client-side Ntor version 3 handshake, part one.

Given a secure rng, a relay’s public key, a secret message to send, and a shared verification string, generate a new handshake state and a message to send to the relay.