encrypt

Function encrypt 

fn encrypt(key: &Zeroizing<[u8; 32]>, m: &[u8]) -> Vec<u8> 
Perform a symmetric encryption operation and return the encrypted data.

(This isn’t safe to do more than once with the same key, but we never do that in this protocol.)