server_handshake_ntor_v3

Function server_handshake_ntor_v3 

fn server_handshake_ntor_v3<RNG: CryptoRng + RngCore, REPLY: MsgReply>(
    rng: &mut RNG,
    reply_fn: &mut REPLY,
    message: &[u8],
    keys: &[NtorV3SecretKey],
    verification: &[u8],
) -> Result<(Vec<u8>, NtorV3XofReader), RelayHandshakeError>
Complete an ntor v3 handshake as a server.

Use the provided rng to generate keys; use the provided reply_fn to handle incoming client secret message and decide how to reply. The client’s handshake is in message. Our private key(s) are in keys. The verification string must match the string provided by the client.

On success, return the server handshake message to send, and an XofReader to use in generating circuit keys.