AuxDataReply

Trait AuxDataReply 

pub(crate) trait AuxDataReply<H>
where H: ServerHandshake + ?Sized,
{ // Required method fn reply(&mut self, msg: &H::ClientAuxData) -> Option<H::ServerAuxData>; }
Trait for an object that handles incoming auxiliary data and returns the server’s auxiliary data to be included in the reply.

This is implemented for FnMut(&H::ClientAuxData) -> Option<H::ServerAuxData> automatically.

Required Methods

fn reply(&mut self, msg: &H::ClientAuxData) -> Option<H::ServerAuxData>

Given a list of extensions received from a client, decide what extensions to send in reply.

Return None if the handshake should fail.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

Source

impl<F, H> AuxDataReply<H> for F