pub type ToplevelAccount = Arc<MemoryQuotaTracker>;Expand description
Account for the whole system
There will typically be only one of these for an entire Arti client or relay.
This is not really an Account.
We don’t want anyone to make a Participant from this,
because if that Participant were reclaimed, everything would be torn down.
We provide the type alias for consistency/readability at call sites.
See the memquota module documentation.
Aliased Type
pub struct ToplevelAccount { /* private fields */ }