#[non_exhaustive]pub enum PeerAddr {
Direct(SocketAddr),
Pt(PtTarget),
}Expand description
Represents the address of a connected peer used for a tor channel.
Clever observer here would see that this is basically a tor_linkspec::ChannelMethod which
has a Direct variant with a vector of address which is incoherent with the semantic of “where we
are connected to”.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Direct(SocketAddr)
The socket address we are directly connected to.
Pt(PtTarget)
The pluggable transport target used to connect.
Note that this is not a specific address but the PT transport details. We keep those
because if the PtTargetAddr happens to be None, we need to compare the transport name
and settings to match a channel.
Implementations
Sourceimpl PeerAddr
impl PeerAddr
Sourcepub(crate) const UNSPECIFIED: Self
pub(crate) const UNSPECIFIED: Self
Unspecified address used for placeholder in unit tests.
Sourcepub fn netinfo_addr(&self) -> Option<IpAddr>
pub fn netinfo_addr(&self) -> Option<IpAddr>
Return the IP address that should be used in the NETINFO cell.
A None value implies we didn’t use an IP address to connect (see PtTarget). The
tor_cell::chancell::msg::Netinfo interface will treat a None as an IPv4 unspecified
address (0.0.0.0).
Sourcepub fn socket_addr(&self) -> Option<SocketAddr>
pub fn socket_addr(&self) -> Option<SocketAddr>
Return the socket address of this peer.
None means that this is a PT and doesn’t use an internet address. One example is it uses a hostname.