pub struct VerifiedInitiatorRelayChannel<T: AsyncRead + AsyncWrite + CertifiedConn + StreamOps + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider> {
inner: VerifiedChannel<T, S>,
auth_material: Arc<RelayChannelAuthMaterial>,
netinfo_cell: Netinfo,
auth_challenge_cell: AuthChallenge,
peer_tls_cert_digest: [u8; 32],
slog_digest: SlogDigest,
my_addrs: Vec<IpAddr>,
create_request_handler: Arc<CreateRequestHandler>,
}Expand description
A verified relay initiator channel.
Holding this object means the channel TLS layer has been verified against the received CERTS cell and we now believe that we are talking to the right relay end point.
The finish() function needs to be called in order to finalize this channel into a generic Channel/Reactor.
Fields
inner: VerifiedChannel<T, S>The common unverified channel that both client and relays use.
auth_material: Arc<RelayChannelAuthMaterial>Relay channel authentication material.
netinfo_cell: NetinfoThe netinfo cell that we got from the relay.
auth_challenge_cell: AuthChallengeThe AUTH_CHALLENGE cell that we got from the relay.
peer_tls_cert_digest: [u8; 32]The peer TLS certificate digest.
slog_digest: SlogDigestThe SLOG digest.
my_addrs: Vec<IpAddr>Our advertised IP addresses.
create_request_handler: Arc<CreateRequestHandler>Provided to each new channel so that they can handle CREATE* requests.
Implementations
Sourceimpl<T, S> VerifiedInitiatorRelayChannel<T, S>where
T: AsyncRead + AsyncWrite + CertifiedConn + StreamOps + Send + Unpin + 'static,
S: CoarseTimeProvider + SleepProvider,
impl<T, S> VerifiedInitiatorRelayChannel<T, S>where
T: AsyncRead + AsyncWrite + CertifiedConn + StreamOps + Send + Unpin + 'static,
S: CoarseTimeProvider + SleepProvider,
Sourcepub async fn finish(
self,
peer_addr: PeerAddr,
) -> Result<(Arc<Channel>, Reactor<S>)>where
S: Runtime,
pub async fn finish(
self,
peer_addr: PeerAddr,
) -> Result<(Arc<Channel>, Reactor<S>)>where
S: Runtime,
Send our msg::Certs, msg::Authenticate and msg::Netinfo to the relay to finish
the handshake, which will create an open channel and reactor.
The resulting channel is considered, by Tor protocol standard, an authenticated relay channel on which circuits can be opened.