UnverifiedResponderRelayChannel

Struct UnverifiedResponderRelayChannel 

pub struct UnverifiedResponderRelayChannel<T: AsyncRead + AsyncWrite + CertifiedConn + StreamOps + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider> {
    pub(crate) inner: UnverifiedChannel<T, S>,
    pub(crate) auth_cell: Authenticate,
    pub(crate) netinfo_cell: Netinfo,
    pub(crate) certs_cell: Certs,
    pub(crate) auth_material: Arc<RelayChannelAuthMaterial>,
    pub(crate) my_addrs: Vec<IpAddr>,
    pub(crate) peer_addr: PeerAddr,
    pub(crate) clog_digest: ClogDigest,
    pub(crate) slog_digest: SlogDigest,
    pub(crate) create_request_handler: Arc<CreateRequestHandler>,
}
A verifiable relay responder channel that is currently unverified. This can only be a relay on the other end.

The verify() and then finish() functions are to be used to get a final Channel/Reactor.

Fields

inner: UnverifiedChannel<T, S>

The common unverified channel that both client and relays use.

auth_cell: Authenticate

AUTHENTICATE cell received from the initiator.

netinfo_cell: Netinfo

The netinfo cell received from the initiator.

certs_cell: Certs

The msg::Certs cell received from the initiator.

auth_material: Arc<RelayChannelAuthMaterial>

Our authentication key material.

my_addrs: Vec<IpAddr>

Our advertised addresses.

peer_addr: PeerAddr

The peer address which we know is a relay.

clog_digest: ClogDigest

The CLOG digest.

slog_digest: SlogDigest

The SLOG digest.

create_request_handler: Arc<CreateRequestHandler>

Provided to each new channel so that they can handle CREATE* requests.

Implementations

Source

impl<T, S> UnverifiedResponderRelayChannel<T, S>

pub fn verify( self, peer_target_no_ids: &OwnedChanTarget, our_tls_cert: &[u8], now: Option<SystemTime>, ) -> Result<VerifiedResponderRelayChannel<T, S>>

Validate the certificates and keys in the relay’s handshake.

‘peer_target_no_ids’ is the peer, without identities as we are accepting a connection and thus don’t have expectations on any identity, that we want to make sure we’re connecting to.

‘our_tls_cert’ is the x.509 certificate that we presented during the TLS handshake.

‘now’ is the time at which to check that certificates are valid. None means to use the current time. It can be used for testing to override the current view of the time.

This is a separate function because it’s likely to be somewhat CPU-intensive.

pub fn clock_skew(&self) -> ClockSkew

Return the clock skew of this channel.