ExtendRequestHandler

Struct ExtendRequestHandler 

pub(super) struct ExtendRequestHandler {
    unique_id: UniqId,
    have_seen_extend2: bool,
    chan_provider: Arc<dyn ChannelProvider<BuildSpec = OwnedChanTarget> + Send + Sync>,
    inbound_peer: Arc<PeerInfo>,
    event_tx: Sender<CircEvent>,
    memquota: CircuitAccount,
}
Helper for handling EXTEND2 cells.

Fields

unique_id: UniqId

An identifier for logging about this handler.

have_seen_extend2: bool

Whether we have received an EXTEND2 on this circuit.

chan_provider: Arc<dyn ChannelProvider<BuildSpec = OwnedChanTarget> + Send + Sync>

A handle to a ChannelProvider, used for initiating outgoing Tor channels.

Note: all circuit reactors of a relay need to be initialized with the same underlying Tor channel provider (ChanMgr), to enable the reuse of existing Tor channels where possible.

inbound_peer: Arc<PeerInfo>

The identity of the inbound relay (the previous hop).

event_tx: Sender<CircEvent>

A stream of events to be read from the main loop of the reactor.

memquota: CircuitAccount

Memory quota account

Implementations

Source

impl ExtendRequestHandler

pub(super) fn new( unique_id: UniqId, chan_provider: Arc<dyn ChannelProvider<BuildSpec = OwnedChanTarget> + Send + Sync>, inbound_peer: Arc<PeerInfo>, event_tx: Sender<CircEvent>, memquota: CircuitAccount, ) -> Self

Create a new ExtendRequestHandler.

pub(super) fn handle_extend2<R: Runtime>( &mut self, runtime: &R, early: bool, msg: UnparsedRelayMsg, ) -> StdResult<(), ReactorError>

Handle an EXTEND2 cell.

This spawns a background task for dealing with the circuit extension, which then reports back the result via the Self::event_tx MPSC stream. Note that this MPSC stream is polled from the ForwardReactor main loop, and each CircEvent is passed back to Forward’s ForwardHandler::handle_event implementation for handling.

async fn extend_circuit<R: Runtime>( _runtime: R, unique_id: UniqId, extend2: Extend2, chan_rx: UnboundedReceiver<ChannelResult>, memquota: CircuitAccount, ) -> StdResult<ExtendResult, ReactorError>

Extend this circuit on the channel received on chan_rx.

Note: this gets spawned in a background task from Self::handle_extend2 so as not to block the reactor main loop.