pub struct CircNetParameters {
pub extend_by_ed25519_id: bool,
pub cc: CongestionControlNetParams,
}Expand description
Network consensus parameters for handling incoming circuits.
Unlike CircParameters,
this is unopinionated and contains all relevant consensus parameters,
which is needed when handling an incoming CREATE* request where the
circuit origin chooses the type/settings
(for example congestion control type) of the circuit.
Fields
extend_by_ed25519_id: boolWhether we should include ed25519 identities when we send EXTEND2 cells.
cc: CongestionControlNetParamsCongestion control network parameters.
Implementations
Sourceimpl CircNetParameters
impl CircNetParameters
Sourcefn as_circ_parameters(
&self,
algorithm: AlgorithmDiscriminants,
) -> Result<CircParameters, Bug>
fn as_circ_parameters( &self, algorithm: AlgorithmDiscriminants, ) -> Result<CircParameters, Bug>
Convert the CircNetParameters into a CircParameters.
We expect the circuit creation handshake to know what congestion control algorithm was
negotiated, and provide that as algorithm.