pub struct CreateRequestHandler {
chan_provider: Weak<dyn ChannelProvider<BuildSpec = OwnedChanTarget> + Send + Sync>,
circ_net_params: RwLock<CircNetParameters>,
ntor_keys: RwLock<RelayNtorKeys>,
}Expand description
Everything needed to handle CREATE* messages on channels.
Fields
chan_provider: Weak<dyn ChannelProvider<BuildSpec = OwnedChanTarget> + Send + Sync>Something that can launch channels. Typically the ChanMgr.
circ_net_params: RwLock<CircNetParameters>Circuit-related network parameters.
ntor_keys: RwLock<RelayNtorKeys>The circuit extension keys.
Implementations
Sourceimpl CreateRequestHandler
impl CreateRequestHandler
Sourcepub fn new(
chan_provider: Weak<dyn ChannelProvider<BuildSpec = OwnedChanTarget> + Send + Sync>,
circ_net_params: CircNetParameters,
ntor_keys: RelayNtorKeys,
) -> Self
pub fn new( chan_provider: Weak<dyn ChannelProvider<BuildSpec = OwnedChanTarget> + Send + Sync>, circ_net_params: CircNetParameters, ntor_keys: RelayNtorKeys, ) -> Self
Build a new CreateRequestHandler.
Sourcepub fn update_params(&self, circ_net_params: CircNetParameters)
pub fn update_params(&self, circ_net_params: CircNetParameters)
Update the circuit parameters from a network consensus.
Sourcepub fn update_ntor_keys(&self, ntor_keys: RelayNtorKeys)
pub fn update_ntor_keys(&self, ntor_keys: RelayNtorKeys)
Update the handler with a new set of circuit extension keys.
This is called periodically by the relay key rotation task.
Sourcepub(crate) fn handle_create<R: Runtime>(
&self,
runtime: &R,
channel: &Arc<Channel>,
our_ed25519_id: &Ed25519Identity,
our_rsa_id: &RsaIdentity,
circ_id: CircId,
msg: &CreateRequest,
memquota: &ChannelAccount,
circ_unique_id: UniqId,
) -> Result<(CreateResponse, RelayCircComponents), Destroy>
pub(crate) fn handle_create<R: Runtime>( &self, runtime: &R, channel: &Arc<Channel>, our_ed25519_id: &Ed25519Identity, our_rsa_id: &RsaIdentity, circ_id: CircId, msg: &CreateRequest, memquota: &ChannelAccount, circ_unique_id: UniqId, ) -> Result<(CreateResponse, RelayCircComponents), Destroy>
Handle a CREATE* cell.
This intentionally does not return a crate::Error so that we don’t accidentally shut
down the channel reactor when we really should be returning a DESTROY. Shutting down a
channel may cause us to leak information about paths of circuits travelling through this
relay. This is especially important here since we’re handling data that is controllable from
the other end of the circuit.
Sourcefn handle_create_inner<R: Runtime>(
&self,
runtime: &R,
channel: &Arc<Channel>,
_our_ed25519_id: &Ed25519Identity,
_our_rsa_id: &RsaIdentity,
circ_id: CircId,
msg: &CreateRequest,
memquota: &ChannelAccount,
circ_unique_id: UniqId,
) -> Result<(CreateResponse, RelayCircComponents), HandleCreateError>
fn handle_create_inner<R: Runtime>( &self, runtime: &R, channel: &Arc<Channel>, _our_ed25519_id: &Ed25519Identity, _our_rsa_id: &RsaIdentity, circ_id: CircId, msg: &CreateRequest, memquota: &ChannelAccount, circ_unique_id: UniqId, ) -> Result<(CreateResponse, RelayCircComponents), HandleCreateError>
See Self::handle_create.
Sourcefn handle_create_fast(
&self,
msg: &CreateFast,
) -> Result<CompletedHandshakeComponents, HandleCreateError>
fn handle_create_fast( &self, msg: &CreateFast, ) -> Result<CompletedHandshakeComponents, HandleCreateError>
The handshake code for a CREATE_FAST request.