StreamFlowCtrl

Struct StreamFlowCtrl 

pub(crate) struct StreamFlowCtrl {
    inner: StreamFlowCtrlInner,
}
Manages the circuit reactor’s flow control for a stream.

Note that the flow control logic can be distributed across multiple parts of Arti. For example some flow control logic will exist in the circuit reactor, but other logic will exist in the stream’s DataStream. So this doesn’t include all flow control logic.

Fields

inner: StreamFlowCtrlInner

Private internal enum.

Implementations

Source

impl StreamFlowCtrl

pub(crate) fn new_window(window: SendWindow<StreamParams>) -> Self

Returns a new sendme-window-based StreamFlowCtrl.

pub(crate) fn new_xon_xoff( params: Arc<FlowCtrlParameters>, use_sidechannel_mitigations: bool, rate_limit_updater: Sender<StreamRateLimit>, drain_rate_requester: NotifySender<DrainRateRequest>, ) -> Self

Returns a new xon/xoff-based StreamFlowCtrl.

pub(crate) fn half_stream(self) -> HalfStreamFlowCtrl

Once this end of the stream is closed and the stream becomes a half-stream (HalfStream), this method will turn the flow control object into a version that is designed to be used for half-streams.