BufferIsEmpty

Trait BufferIsEmpty 

pub(crate) trait BufferIsEmpty {
    // Required method
    fn is_empty(self: Pin<&mut Self>) -> bool;
}
Used by the XonXoffReader to decide when to send a drain rate update (typically resulting in an XON message).

Required Methods

fn is_empty(self: Pin<&mut Self>) -> bool

Returns true if there are no incoming bytes buffered on this stream.

This takes a &mut so that implementers can unobtrusive_peek() a stream if necessary.

Implementors