pub(crate) struct XonXoffFlowCtrl {
params: Arc<FlowCtrlParameters>,
rate_limit_updater: Sender<StreamRateLimit>,
drain_rate_requester: NotifySender<DrainRateRequest>,
last_sent_xon_xoff: Option<XonXoffMsg>,
xoff_limit: CellCount<{ _ }>,
sidechannel_mitigation: Option<SidechannelMitigation>,
}Expand description
State for XON/XOFF flow control.
Fields
params: Arc<FlowCtrlParameters>Consensus parameters.
rate_limit_updater: Sender<StreamRateLimit>How we communicate rate limit updates to the
DataWriter.
drain_rate_requester: NotifySender<DrainRateRequest>How we communicate requests for new drain rate updates to the
XonXoffReader.
last_sent_xon_xoff: Option<XonXoffMsg>The last rate limit we sent.
xoff_limit: CellCount<{ _ }>The buffer limit at which we should send an XOFF.
In prop324 it says that this will be either cc_xoff_client or cc_xoff_exit depending on
whether we’re a client/hs or exit, but we deviate from the spec here (see how it is set
below).
sidechannel_mitigation: Option<SidechannelMitigation>DropMark sidechannel mitigations.
This is only enabled if we are a client (including an onion service).
Implementations
Sourceimpl XonXoffFlowCtrl
impl XonXoffFlowCtrl
Sourcepub(crate) fn new(
params: Arc<FlowCtrlParameters>,
use_sidechannel_mitigations: bool,
rate_limit_updater: Sender<StreamRateLimit>,
drain_rate_requester: NotifySender<DrainRateRequest>,
) -> Self
pub(crate) fn new( params: Arc<FlowCtrlParameters>, use_sidechannel_mitigations: bool, rate_limit_updater: Sender<StreamRateLimit>, drain_rate_requester: NotifySender<DrainRateRequest>, ) -> Self
Returns a new xon/xoff-based state.