StreamQueueSender

Struct StreamQueueSender 

pub(crate) struct StreamQueueSender {
    sender: Sender<UnparsedRelayMsg, MpscUnboundedSpec>,
    counter: Arc<Mutex<usize>>,
}
The sending end of a channel of incoming stream messages.

Fields

sender: Sender<UnparsedRelayMsg, MpscUnboundedSpec>

The inner sender.

counter: Arc<Mutex<usize>>

Number of bytes within the queue.

Implementations

Source

impl StreamQueueSender

pub(crate) fn project<'pin>( self: Pin<&'pin mut Self>, ) -> __StreamQueueSenderProjection<'pin>

pub(crate) fn project_ref<'pin>( self: Pin<&'pin Self>, ) -> __StreamQueueSenderProjectionRef<'pin>

Source

impl StreamQueueSender

pub(crate) fn approx_stream_bytes(&self) -> usize

Get the approximate number of data bytes queued for this stream.

As messages can be dequeued at any time, the return value may be larger than the actual number of bytes queued for this stream.