pub(crate) struct HalfStream {
flow_control: StreamFlowCtrl,
recvw: RecvWindow<StreamParams>,
cmd_checker: Box<dyn CmdChecker + Send + 'static>,
}Expand description
Type to track state of half-closed streams.
A half-closed stream is one where we’ve sent an END cell, but where the other side might still send us data.
We need to track these streams instead of forgetting about them entirely, since otherwise we’d be vulnerable to a class of “DropMark” attacks; see https://gitlab.torproject.org/tpo/core/tor/-/issues/25573.
Fields
flow_control: StreamFlowCtrlFlow control for this stream.
Used to process incoming flow control messages (SENDME, XON, etc).
recvw: RecvWindow<StreamParams>Receive window for this stream. Used to detect whether we get too many data cells.
cmd_checker: Box<dyn CmdChecker + Send + 'static>Object to tell us which cells to accept on this stream.
Implementations
Sourceimpl HalfStream
impl HalfStream
Sourcepub(crate) fn new(
flow_control: StreamFlowCtrl,
recvw: RecvWindow<StreamParams>,
cmd_checker: Box<dyn CmdChecker + Send + 'static>,
) -> Self
pub(crate) fn new( flow_control: StreamFlowCtrl, recvw: RecvWindow<StreamParams>, cmd_checker: Box<dyn CmdChecker + Send + 'static>, ) -> Self
Create a new half-closed stream.
Sourcepub(crate) fn handle_msg(
&mut self,
msg: UnparsedRelayMsg,
) -> Result<StreamStatus>
pub(crate) fn handle_msg( &mut self, msg: UnparsedRelayMsg, ) -> Result<StreamStatus>
Process an incoming message and adjust this HalfStream accordingly. Give an error if the protocol has been violated.
The caller must handle END cells; it is an internal error to pass END cells to this method. no ends here.