pub struct RelayChannelAuthMaterial {
pub(crate) rsa_id_der_digest: [u8; 32],
pub(crate) rsa_id: RsaIdentity,
pub(crate) ed_id: Ed25519Identity,
pub(crate) link_sign_kp: RelayLinkSigningKeypair,
pub(crate) cert_id_sign_ed: EncodedEd25519Cert,
pub(crate) cert_sign_tls_ed: EncodedEd25519Cert,
pub(crate) cert_sign_link_auth_ed: EncodedEd25519Cert,
pub(crate) cert_id_x509_rsa: Vec<u8>,
pub(crate) cert_id_rsa: EncodedRsaCrosscert,
pub(crate) tls_key_and_cert: TlsKeyAndCert,
}Expand description
Object containing the keys and certificates for channel authentication.
We use this intermediary object in order to not have tor-proto crate have access to the KeyMgr meaning access to all keys. This restricts the view to what is needed.
Fields
rsa_id_der_digest: [u8; 32]The SHA256(DER(KP_relayid_rsa)) digest for the AUTHENTICATE cell CID.
rsa_id: RsaIdentityOur RSA identity KP_relayid_rsa (SHA1). Needed for HasRelayIds which is required to
compare this with a tor_linkspec::ChanTarget.
ed_id: Ed25519IdentityOur Ed identity key (KP_relayid_ed). For the msg::Authenticate cell CID_ED field.
link_sign_kp: RelayLinkSigningKeypairOur link signing keypair. Used to sign the msg::Authenticate cell.
cert_id_sign_ed: EncodedEd25519CertThe Ed25519 identity signing cert (CertType 4) for the msg::Certs cell.
cert_sign_tls_ed: EncodedEd25519CertThe Ed25519 signing TLS cert (CertType 5) for the msg::Certs cell.
cert_sign_link_auth_ed: EncodedEd25519CertThe Ed25519 signing link auth cert (CertType 6) for the msg::Certs cell.
cert_id_x509_rsa: Vec<u8>Legacy: the RSA identity X509 cert (CertType 2) for the msg::Certs cell.
We only have the bytes here as create_legacy_rsa_id_cert() takes a key and gives us back the encoded cert.
cert_id_rsa: EncodedRsaCrosscertLegacy: the RSA identity cert (CertType 7) for the msg::Certs cell.
tls_key_and_cert: TlsKeyAndCertTls key and cert. This is for the TLS acceptor object needed to be a responder (TLS server side).
Implementations
Sourceimpl RelayChannelAuthMaterial
impl RelayChannelAuthMaterial
Sourcepub fn new(
rsa_id_pk: &PublicKey,
ed_id: Ed25519Identity,
link_sign_kp: RelayLinkSigningKeypair,
cert_id_sign_ed: EncodedEd25519Cert,
cert_sign_tls_ed: EncodedEd25519Cert,
cert_sign_link_auth_ed: EncodedEd25519Cert,
cert_id_x509_rsa: Vec<u8>,
cert_id_rsa: EncodedRsaCrosscert,
tls_key_and_cert: TlsKeyAndCert,
) -> Self
pub fn new( rsa_id_pk: &PublicKey, ed_id: Ed25519Identity, link_sign_kp: RelayLinkSigningKeypair, cert_id_sign_ed: EncodedEd25519Cert, cert_sign_tls_ed: EncodedEd25519Cert, cert_sign_link_auth_ed: EncodedEd25519Cert, cert_id_x509_rsa: Vec<u8>, cert_id_rsa: EncodedRsaCrosscert, tls_key_and_cert: TlsKeyAndCert, ) -> Self
Constructor.
Sourcepub fn tls_key_and_cert(&self) -> &TlsKeyAndCert
pub fn tls_key_and_cert(&self) -> &TlsKeyAndCert
Return the TLS key and certificate to use for the underlying TLS provider.
This is used by the TLS acceptor that acts as the TLS server provider.
Sourcepub(crate) fn ed_id_bytes(&self) -> [u8; 32]
pub(crate) fn ed_id_bytes(&self) -> [u8; 32]
Return our Ed identity key (KP_relayid_ed) as bytes.