RelayIdentities

Struct RelayIdentities 

pub struct RelayIdentities { /* private fields */ }
Available on crate feature relay only.
Object containing the key and certificate that basically identifies us as a relay. They are used for channel authentication.

We use this intermediary object in order to not have tor-proto crate have access to the KeyMgr meaning access to all keys. This restricts the view to what is needed.

Implementations

Source

impl RelayIdentities

pub fn new( rsa_id_pk: &PublicKey, ed_id: Ed25519Identity, link_sign_kp: RelayLinkSigningKeypair, cert_id_sign_ed: EncodedEd25519Cert, cert_sign_tls_ed: EncodedEd25519Cert, cert_sign_link_auth_ed: EncodedEd25519Cert, cert_id_x509_rsa: Vec<u8>, cert_id_rsa: EncodedRsaCrosscert, tls_key_and_cert: TlsKeyAndCert, ) -> Self

Constructor.

pub fn tls_key_and_cert(&self) -> &TlsKeyAndCert

Return the TLS key and certificate to use for the underlying TLS provider.

This is used by the TLS acceptor that acts as the TLS server provider.