pub(crate) fn fake_mq<A: SpecificAccount>() -> AExpand description
Convenience alias for
memquota::SpecificAccount::new_noop()
Available only in tests, which makes diff hunks which call this more obviously correct.
pub(crate) fn fake_mq<A: SpecificAccount>() -> AConvenience alias for
memquota::SpecificAccount::new_noop()
Available only in tests, which makes diff hunks which call this more obviously correct.