pub(crate) struct CountingPolicy {
remaining: Option<NonMaxU32>,
}Expand description
A sink-blocking Policy that can allow a limited number of items to be sent.
This policy may be in three states:
- Completely blocked
- Completely unblocked: Able to send an unlimited number of items.
- Will become blocked after a certain number of items are sent.
Fields
remaining: Option<NonMaxU32>The number of items that may currently be sent.
None represents an unlimited number.
Implementations
Sourceimpl CountingPolicy
impl CountingPolicy
Sourcepub(crate) fn new_unlimited() -> Self
pub(crate) fn new_unlimited() -> Self
Return a new unlimited CountingPolicy.
Sourcepub(crate) fn new_blocked() -> Self
pub(crate) fn new_blocked() -> Self
Return a new completely blocked CountingPolicy.
Sourcepub(crate) fn new_limited(n: u32) -> Self
pub(crate) fn new_limited(n: u32) -> Self
Return a new CountingPolicy that allows n items, and then becomes blocked.
Limitations:
If n is greater than MAX_LIMIT, only MAX_LIMIT items will be allowed.
Sourcefn saturating_add(&self, n: u32) -> Self
fn saturating_add(&self, n: u32) -> Self
Return a new CountingPolicy that allows up to n more items to be sent
than this one.
Limitations:
If the total number of allowed items would be greater than MAX_LIMIT,
only MAX_LIMIT items will be allowed.