DynamicRateLimitedWriter

Struct DynamicRateLimitedWriter 

pub(crate) struct DynamicRateLimitedWriter<W: AsyncWrite, S, P: SleepProvider> {
    writer: RateLimitedWriter<W, P>,
    updates: S,
}
A rate-limited async writer.

This wraps a RateLimitedWriter and watches a stream for configuration changes (such as rate limit changes).

Fields

writer: RateLimitedWriter<W, P>

The rate-limited writer.

updates: S

A stream that provides configuration updates, including rate limit updates.

Implementations

Source

impl<W: AsyncWrite, S, P: SleepProvider> DynamicRateLimitedWriter<W, S, P>

pub(crate) fn project<'pin>( self: Pin<&'pin mut Self>, ) -> __DynamicRateLimitedWriterProjection<'pin, W, S, P>

pub(crate) fn project_ref<'pin>( self: Pin<&'pin Self>, ) -> __DynamicRateLimitedWriterProjectionRef<'pin, W, S, P>

Source

impl<W, S, P> DynamicRateLimitedWriter<W, S, P>

pub(crate) fn new(writer: RateLimitedWriter<W, P>, updates: S) -> Self

Create a new DynamicRateLimitedWriter.

This wraps the writer and watches for configuration changes from the updates stream.

pub(crate) fn inner(&self) -> &W

Access the inner AsyncWrite writer of the RateLimitedWriter.