pub(crate) struct RateLimitedWriterConfig {
pub(crate) rate: u64,
pub(crate) burst: u64,
pub(crate) wake_when_bytes_available: NonZero<u64>,
}Expand description
The refill rate and burst for a RateLimitedWriter.
Fields
rate: u64The refill rate in bytes/second.
burst: u64The “burst” in bytes.
wake_when_bytes_available: NonZero<u64>When polled, block until at most this many bytes are available.
Or in other words, wake when we can write this many bytes, even if the provided buffer is larger.
For example if a user attempts to write a large buffer, we usually don’t want to block until the entire buffer can be written. We’d prefer several partial writes to a single large write. So instead of blocking until the entire buffer can be written, we only block until at most this many bytes are available.