pub(crate) struct AtomicOptTimestamp {
latest: AtomicU64,
}Expand description
An object for determining whether an event happened, and if yes, when it happened.
Every Timestamp has internal mutability. A timestamp can move
forward in time, but never backwards.
Internally, it uses the coarsetime crate to represent times in a way
that lets us do atomic updates.
Fields
latest: AtomicU64A timestamp (from coarsetime) describing when this timestamp
was last updated.
I’d rather just use coarsetime::Instant, but that doesn’t have
an atomic form.
Implementations
Sourceimpl AtomicOptTimestamp
impl AtomicOptTimestamp
Sourcepub(crate) fn update_if_none(&self)
pub(crate) fn update_if_none(&self)
If the timestamp is currently unset, then set it to the current time. Otherwise leave it alone.
Sourcepub(crate) fn time_since_update(&self) -> Option<Duration>
pub(crate) fn time_since_update(&self) -> Option<Duration>
Return the time since update was last called.
Return None if update was never called.
Sourcepub(crate) fn time_since_update_at(&self, now: Instant) -> Option<Duration>
pub(crate) fn time_since_update_at(&self, now: Instant) -> Option<Duration>
Return the time between the time when update was last
called, and the time now.
Return None if update was never called, or now is before
that time.