TunnelActivity

Struct TunnelActivity 

pub(crate) struct TunnelActivity {
    inner: Inner,
}
An object to track whether a tunnel or circuit should still be considered active.

The “active” status of a tunnel depends on whether it is in use for streams, and if not, how much time has passed since it was last in use for streams.

Ordering and aggregation

We rely on the ordering for TunnelActivity structs. In particular, we depend on the property that a “more active” TunnelActivity is greater than a less active one.

Specifically,

  • a TunnelActivity with streams is “more active” than one without streams.
  • a TunnelActivity that was last used for streams recently is “more active” than one that was last used for streams more time ago.
  • a TunnelActivity that was ever in use for streams is “more active” than one that has never been used.

For implementation convenience, we do not generally keep a TunnelActivity for an entire tunnel. Instead, we keep a separate TunnelActivity for each hop of each circuit. When we need to find the TunnelActivity of the tunnel as a whole, we look for the TunnelActivity of the “most active” hop. This does not give an accurate count of all the streams on the tunnel, but we don’t generally care about that.

We could instead add the TunnelActivity for each hop, but that would be a bit more implementation effort to little benefit, and we’d need to avoid counting conflux join points twice.

Fields

inner: Inner

Actual activity for the associated tunnel.

Implementations

Source

impl TunnelActivity

pub(crate) fn never_used() -> Self

Construct a new TunnelActivity for a tunnel that has never been used.

pub(crate) fn inc_streams(&mut self) -> InTunnelActivity

Increase the number of streams on this tunnel by one.

pub(crate) fn dec_streams(&mut self, token: InTunnelActivity)

Decrease the number of streams on this tunnel by one.

pub(crate) fn n_open_streams(&self) -> usize

Return the number of open streams on this tunnel.

(But see note on TunnelActivity documentation)

pub(crate) fn disused_since(&self) -> Option<Instant>

Return the time at which this tunnel was last in use.

Returns None if the tunnel has open streams right now, or if it has never had any open streams.

A note about time

The returned Instant value is a direct result of an earlier call to Instant::get(). It is not affected by any runtime mocking.