InboundClientLayer

Trait InboundClientLayer 

pub trait InboundClientLayer {
    // Required method
    fn decrypt_inbound(
        &mut self,
        cmd: ChanCmd,
        cell: &mut RelayCellBody,
    ) -> Option<SendmeTag>;
}
A client’s view of the crypto state shared with a single relay on a circuit, as used for inbound cells.

Required Methods

fn decrypt_inbound( &mut self, cmd: ChanCmd, cell: &mut RelayCellBody, ) -> Option<SendmeTag>

Decrypt a CellBody that passed through this layer.

Return an authentication tag if this layer is the originator.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

Source

impl<EtBC, PrfBC> InboundClientLayer for tor_proto::bench_utils::cgo::ClientInbound<EtBC, PrfBC>
where EtBC: BlkCipherDec, PrfBC: BlkCipherEnc,

Source

impl<L: OutboundRelayLayer> InboundClientLayer for ResponderInboundLayer<L>

Source

impl<SC: StreamCipher, D: Digest + Clone> InboundClientLayer for tor_proto::bench_utils::tor1::ClientInbound<SC, D>