pub trait InboundClientLayer {
// Required method
fn decrypt_inbound(
&mut self,
cmd: ChanCmd,
cell: &mut RelayCellBody,
) -> Option<SendmeTag>;
}Available on crate feature
bench only.Expand description
A client’s view of the crypto state shared with a single relay on a circuit, as used for inbound cells.
Required Methods
Sourcefn decrypt_inbound(
&mut self,
cmd: ChanCmd,
cell: &mut RelayCellBody,
) -> Option<SendmeTag>
fn decrypt_inbound( &mut self, cmd: ChanCmd, cell: &mut RelayCellBody, ) -> Option<SendmeTag>
Decrypt a CellBody that passed through this layer.
Return an authentication tag if this layer is the originator.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".
Implementors
impl<EtBC, PrfBC> InboundClientLayer for tor_proto::bench_utils::cgo::ClientInbound<EtBC, PrfBC>where
EtBC: BlkCipherDec,
PrfBC: BlkCipherEnc,
Available on crate feature
counter-galois-onion only.