pub(super) struct CircMap {
m: HashMap<CircId, CircEnt>,
range: CircIdRange,
open_count: usize,
}Expand description
A map from circuit IDs to circuit entries. Each channel has one.
Fields
m: HashMap<CircId, CircEnt>Map from circuit IDs to entries
range: CircIdRangeRule for allocating new circuit IDs.
open_count: usizeNumber of open or opening entry in this map.
Implementations
Sourceimpl CircMap
impl CircMap
Sourcepub(super) fn new(idrange: CircIdRange) -> Self
pub(super) fn new(idrange: CircIdRange) -> Self
Make a new empty CircMap
Sourcepub(super) fn add_origin_ent<R: Rng>(
&mut self,
rng: &mut R,
createdsink: Sender<CreateResponse>,
sink: Sender<AnyChanMsg, MpscSpec>,
padding_ctrl: PaddingController,
) -> Result<CircId>
pub(super) fn add_origin_ent<R: Rng>( &mut self, rng: &mut R, createdsink: Sender<CreateResponse>, sink: Sender<AnyChanMsg, MpscSpec>, padding_ctrl: PaddingController, ) -> Result<CircId>
Add a new set of elements (corresponding to a
PendingClientTunnel)
as an entry to this map.
On success return the allocated circuit ID.
Sourcepub(super) fn add_relay_ent(
&mut self,
circ_id: CircId,
circ: Arc<RelayCirc>,
sink: Sender<AnyChanMsg, MpscSpec>,
padding_ctrl: PaddingController,
) -> StdResult<(), DestroyReason>
pub(super) fn add_relay_ent( &mut self, circ_id: CircId, circ: Arc<RelayCirc>, sink: Sender<AnyChanMsg, MpscSpec>, padding_ctrl: PaddingController, ) -> StdResult<(), DestroyReason>
Add a new set of elements (corresponding to a RelayCirc) as an entry to this map.
We use DestroyReason as the return type since we very likely want to destroy the circuit
if this fails, and not return an error and destroy the entire channel.
Sourcepub(super) fn get_mut(&mut self, id: CircId) -> Option<MutCircEnt<'_>>
pub(super) fn get_mut(&mut self, id: CircId) -> Option<MutCircEnt<'_>>
Return the entry for id in this map, if any.
Sourcepub(super) fn note_cell_flushed(
&mut self,
id: CircId,
info: QueuedCellPaddingInfo,
)
pub(super) fn note_cell_flushed( &mut self, id: CircId, info: QueuedCellPaddingInfo, )
Inform the relevant circuit’s padding subsystem that a given cell has been flushed.
Sourcepub(super) fn advance_from_opening(
&mut self,
id: CircId,
) -> Result<Sender<CreateResponse>>
pub(super) fn advance_from_opening( &mut self, id: CircId, ) -> Result<Sender<CreateResponse>>
See whether ‘id’ is an opening circuit. If so, mark it “open” and return a oneshot::Sender that is waiting for its create cell.
Sourcepub(super) fn destroy_sent(&mut self, id: CircId, hs: HalfCirc)
pub(super) fn destroy_sent(&mut self, id: CircId, hs: HalfCirc)
Called when we have sent a DESTROY on a circuit. Configures a “HalfCirc” object to track how many cells we get on this circuit, and to prevent us from reusing it immediately.
Sourcepub(super) fn remove(&mut self, id: CircId) -> Option<CircEnt>
pub(super) fn remove(&mut self, id: CircId) -> Option<CircEnt>
Extract the value from this map with ‘id’ if any
Sourcepub(super) fn open_ent_count(&self) -> usize
pub(super) fn open_ent_count(&self) -> usize
Return the total number of open and opening entries in the map