pub(crate) fn new_frame<T, I>(tls: T, ty: I) -> Framed<T, ChannelCellHandler>Expand description
Helper: Return a new channel frame ChannelFrame from an object implementing AsyncRead + AsyncWrite. In the tor context, it is always a TLS stream.
The ty (type) argument needs to be able to transform into a handler::ChannelCellHandler which would generally be a ChannelType.