fn decode_as_any<R>(
stage: &MessageStage,
codec: &mut ChannelCodec,
src: &mut BytesMut,
) -> Result<Option<AnyChanCell>, Error>Expand description
Helper function to decode a cell within a restricted msg set into an AnyChanCell.
The given stage is used to know which error to return.