fn encode_as_any<R>(
stage: &MessageStage,
cell: AnyChanCell,
codec: &mut ChannelCodec,
dst: &mut BytesMut,
) -> Result<(), Error>Expand description
Helper function to encode an AnyChanCell cell that is within a restricted msg set R.
The given stage is used to know which error to return.