pub(super) fn decode_cell(
chan_type: ChannelType,
stage: &MessageStage,
codec: &mut ChannelCodec,
src: &mut BytesMut,
) -> Result<Option<(AnyChanCell, Bytes)>, Error>Expand description
Decode cell using the given channel type, message stage, codec and byte source.
Returns both the cell and the bytes that the cell were decoded from.