pub(crate) struct ChannelSender {
cell_tx: CountingSink<Sender<(AnyChanCell, Option<QueuedCellPaddingInfo>), MpscSpec>>,
reactor_closed_rx: Receiver<Result<CloseInfo>>,
unique_id: UniqId,
padding_ctrl: PaddingController,
}Expand description
A handle to a Channel` that can be used, by circuits, to send channel cells.
Fields
cell_tx: CountingSink<Sender<(AnyChanCell, Option<QueuedCellPaddingInfo>), MpscSpec>>MPSC sender to send cells.
reactor_closed_rx: Receiver<Result<CloseInfo>>A receiver used to check if the channel is closed.
unique_id: UniqIdUnique ID for this channel. For logging.
padding_ctrl: PaddingControllerPadding controller for this channel: used to report when we queue data that will eventually wind up on the channel.
Implementations
Sourceimpl ChannelSender
impl ChannelSender
Sourcefn check_cell(&self, cell: &AnyChanCell) -> Result<()>
fn check_cell(&self, cell: &AnyChanCell) -> Result<()>
Check whether a cell type is permissible to be sent on an open client channel.
Sourcepub(crate) fn time_provider(&self) -> &DynTimeProvider
pub(crate) fn time_provider(&self) -> &DynTimeProvider
Obtain a reference to the ChannelSender’s DynTimeProvider
(This can sometimes be used to avoid having to keep a separate clone of the time provider.)
Sourcepub(crate) fn approx_count(&self) -> usize
pub(crate) fn approx_count(&self) -> usize
Return an approximate count of the number of outbound cells queued for this channel.
This count is necessarily approximate, because the underlying count can be modified by other senders and receivers between when this method is called and when its return value is used.
Does not include cells that have already been passed to the TLS connection.
Circuit padding uses this count to determine when messages are already outbound for the first hop of a circuit.
Sourcepub(crate) fn note_cell_queued(&self)
pub(crate) fn note_cell_queued(&self)
Note that a cell has been queued that will eventually be placed onto this sender.
We use this as an input for padding machines.