pub(crate) type ChannelFrame<T> = Framed<T, ChannelCellHandler>;Expand description
A channel cell frame used for sending and receiving cells on a channel. The handler takes care of the cell codec transition depending in which state the channel is.
ChannelFrame is used to basically handle all in and outbound cells on a channel for its entire lifetime.
Aliased Type
pub(crate) struct ChannelFrame<T> { /* private fields */ }