pub(crate) struct CircuitCellSender {
sink: SinkBlocker<SometimesUnboundedSink<(AnyChanCell, Option<QueuedCellPaddingInfo>), ChannelSender>, BooleanPolicy>,
}Expand description
A sink that a circuit uses to send cells onto a Channel.
(This is a separate type so we can more easily control access to its internals.)
You must poll this type
This type is based on SometimesUnboundedSink.
For queued items to be delivered,
SometimesUnboundedSink must be polled,
even if you don’t have an item to send.
The same rule applies here.
Currently Sink::poll_flush, Sink::poll_close, and Sink::poll_ready
will all work for this purpose.
Fields
sink: SinkBlocker<SometimesUnboundedSink<(AnyChanCell, Option<QueuedCellPaddingInfo>), ChannelSender>, BooleanPolicy>The actual inner sink on which we’ll be sending cells.
See type alias documentation for full details.
Implementations
Sourceimpl CircuitCellSender
impl CircuitCellSender
Sourcepub(crate) fn from_channel_sender(inner: ChannelSender) -> Self
pub(crate) fn from_channel_sender(inner: ChannelSender) -> Self
Construct a new CircuitCellSender to deliver cells onto inner.
Sourcepub(crate) fn n_queued(&self) -> usize
pub(crate) fn n_queued(&self) -> usize
Return the number of cells queued in this Sender that have not yet been flushed onto the channel.
Sourcepub(crate) fn have_queued_cell_for_hop_or_later(&self, hop: HopNum) -> bool
pub(crate) fn have_queued_cell_for_hop_or_later(&self, hop: HopNum) -> bool
Return true if we have a queued cell for the specified hop or later.
Sourcepub(crate) async fn send_unbounded(
&mut self,
entry: (AnyChanCell, Option<QueuedCellPaddingInfo>),
) -> Result<()>
pub(crate) async fn send_unbounded( &mut self, entry: (AnyChanCell, Option<QueuedCellPaddingInfo>), ) -> Result<()>
Send a cell on this sender, even if the underlying channel queues are all full.
You must .await this, but it will never block.
(Its future is always Ready.)
See note on CircuitCellSender type about polling:
If you don’t poll this sink, then queued items might never flush.
Sourcepub(crate) fn time_provider(&self) -> &DynTimeProvider
pub(crate) fn time_provider(&self) -> &DynTimeProvider
Return the time provider used by the underlying channel sender for memory quota purposes.
Sourcepub(crate) fn start_blocking(&mut self)
pub(crate) fn start_blocking(&mut self)
Circpadding only: Put this sink into a blocked state.
When we are blocked, attempts to send() to this sink will fail.
You can still queue items with send_unbounded(),
and they will be sent immediately.
Sourcepub(crate) fn stop_blocking(&mut self)
pub(crate) fn stop_blocking(&mut self)
Circpadding only: Put this sink into an unblocked state.
Sourcepub(crate) async fn congestion_signals(&mut self) -> CongestionSignals
pub(crate) async fn congestion_signals(&mut self) -> CongestionSignals
Note: This is only async because we need a Context to check the underlying sink for readiness. This will register a new waker (or overwrite any existing waker).
Sourcefn sometimes_unbounded(
&self,
) -> &SometimesUnboundedSink<(AnyChanCell, Option<QueuedCellPaddingInfo>), ChannelSender>
fn sometimes_unbounded( &self, ) -> &SometimesUnboundedSink<(AnyChanCell, Option<QueuedCellPaddingInfo>), ChannelSender>
Helper: return a reference to the internal SometimesUnboundedSink
that this CircuitCellSender is based on.
Sourcefn sometimes_unbounded_mut(
&mut self,
) -> &mut SometimesUnboundedSink<(AnyChanCell, Option<QueuedCellPaddingInfo>), ChannelSender>
fn sometimes_unbounded_mut( &mut self, ) -> &mut SometimesUnboundedSink<(AnyChanCell, Option<QueuedCellPaddingInfo>), ChannelSender>
Helper: return a mutable reference to the internal SometimesUnboundedSink
that this CircuitCellSender is based on.
Sourcefn chan_sender(&self) -> &ChannelSender
fn chan_sender(&self) -> &ChannelSender
Helper: Return a reference to the internal ChannelSender
that this CircuitCellSender is based on.
Sourcefn chan_sender_mut(&mut self) -> &mut ChannelSender
fn chan_sender_mut(&mut self) -> &mut ChannelSender
Helper: Return a mutable reference to the internal ChannelSender
that this CircuitCellSender is based on.
Sourcefn pre_queue_blocker_mut(
&mut self,
) -> &mut SinkBlocker<SometimesUnboundedSink<(AnyChanCell, Option<QueuedCellPaddingInfo>), ChannelSender>, BooleanPolicy>
fn pre_queue_blocker_mut( &mut self, ) -> &mut SinkBlocker<SometimesUnboundedSink<(AnyChanCell, Option<QueuedCellPaddingInfo>), ChannelSender>, BooleanPolicy>
Helper: Return a mutable reference to our outer SinkBlocker