pub(super) struct ForwardReactor<R: Runtime, F: ForwardHandler> {
runtime: R,
unique_id: UniqId,
inner: F,
command_rx: UnboundedReceiver<CtrlCmd<F::CtrlCmd>>,
control_rx: UnboundedReceiver<CtrlMsg<F::CtrlMsg>>,
inbound_chan_rx: Receiver<AnyChanMsg, MpscSpec>,
backward_reactor_tx: Sender<BackwardReactorCmd>,
hop_mgr: HopMgr<R>,
circ_events: Receiver<F::CircEvent>,
padding_ctrl: PaddingController,
}Expand description
The forward circuit reactor.
See the reactor module-level docs.
Shuts downs down if an error occurs, or if either the Reactor
or the BackwardReactor shuts down:
- if the
Reactorshuts down, we are alerted via the ctrl/command mpsc channels (their sending ends will close, which causes run_once() to return ReactorError::Shutdown) - if
BackwardReactorshuts down, theReactorwill notice and will itself shut down, which, in turn, causes theForwardReactorto shut down as described above
Fields
runtime: RA handle to the runtime.
unique_id: UniqIdAn identifier for logging about this reactor’s circuit.
inner: FImplementation-dependent part of the reactor.
This enables us to customize the behavior of the reactor, depending on whether we are a client or a relay.
command_rx: UnboundedReceiver<CtrlCmd<F::CtrlCmd>>Channel for receiving control commands.
control_rx: UnboundedReceiver<CtrlMsg<F::CtrlMsg>>Channel for receiving control messages.
inbound_chan_rx: Receiver<AnyChanMsg, MpscSpec>The reading end of the inbound Tor channel.
Yields cells moving from the client towards the exit, if we are a relay, or cells moving towards us, if we are a client.
backward_reactor_tx: Sender<BackwardReactorCmd>Sender for sending commands to the BackwardReactor.
Used for sending:
- circuit-level SENDMEs received from the other endpoint
(
[BackwardReactorCmd::HandleSendme]) - circuit-level SENDMEs that need to be delivered to the other endpoint
(using
[BackwardReactorCmd::SendRelayMsg])
The receiver is in BackwardReactor, which is responsible for
sending cell over the inbound channel.
hop_mgr: HopMgr<R>Hop manager, storing per-hop state, and handles to the stream reactors.
Contains the CircHopList.
circ_events: Receiver<F::CircEvent>An implementation-specific event stream.
Polled from the main loop of the reactor.
Each event is passed to ForwardHandler::handle_event.
padding_ctrl: PaddingControllerA padding controller to which padding-related events should be reported.
Implementations
Sourceimpl<R: Runtime, F: ForwardHandler> ForwardReactor<R, F>
impl<R: Runtime, F: ForwardHandler> ForwardReactor<R, F>
Sourceimpl<R: Runtime, F: ForwardHandler> ForwardReactor<R, F>
impl<R: Runtime, F: ForwardHandler> ForwardReactor<R, F>
Sourcepub(super) fn new(
runtime: R,
unique_id: UniqId,
inner: F,
hop_mgr: HopMgr<R>,
inbound_chan_rx: Receiver<AnyChanMsg, MpscSpec>,
control_rx: UnboundedReceiver<CtrlMsg<F::CtrlMsg>>,
command_rx: UnboundedReceiver<CtrlCmd<F::CtrlCmd>>,
backward_reactor_tx: Sender<BackwardReactorCmd>,
circ_events: Receiver<F::CircEvent>,
padding_ctrl: PaddingController,
) -> Self
pub(super) fn new( runtime: R, unique_id: UniqId, inner: F, hop_mgr: HopMgr<R>, inbound_chan_rx: Receiver<AnyChanMsg, MpscSpec>, control_rx: UnboundedReceiver<CtrlMsg<F::CtrlMsg>>, command_rx: UnboundedReceiver<CtrlCmd<F::CtrlCmd>>, backward_reactor_tx: Sender<BackwardReactorCmd>, circ_events: Receiver<F::CircEvent>, padding_ctrl: PaddingController, ) -> Self
Create a new ForwardReactor.
Sourcefn handle_cmd(
&mut self,
cmd: CtrlCmd<F::CtrlCmd>,
) -> StdResult<(), ReactorError>
fn handle_cmd( &mut self, cmd: CtrlCmd<F::CtrlCmd>, ) -> StdResult<(), ReactorError>
Handle a control command.
Sourcefn handle_msg(
&mut self,
msg: CtrlMsg<F::CtrlMsg>,
) -> StdResult<(), ReactorError>
fn handle_msg( &mut self, msg: CtrlMsg<F::CtrlMsg>, ) -> StdResult<(), ReactorError>
Handle a control message.
Sourcefn note_relay_cell_received(
&self,
hopnum: Option<HopNum>,
c_t_w: bool,
) -> Result<(RelayCellFormat, bool)>
fn note_relay_cell_received( &self, hopnum: Option<HopNum>, c_t_w: bool, ) -> Result<(RelayCellFormat, bool)>
Note that we have received a RELAY cell.
Updates the padding and CC state.
Sourceasync fn handle_relay_cell(
&mut self,
decode_res: RelayCellDecoderResult,
early: bool,
hopnum: Option<HopNum>,
tag: SendmeTag,
) -> StdResult<(), ReactorError>
async fn handle_relay_cell( &mut self, decode_res: RelayCellDecoderResult, early: bool, hopnum: Option<HopNum>, tag: SendmeTag, ) -> StdResult<(), ReactorError>
Handle a RELAY cell.
Sourceasync fn handle_relay_msg(
&mut self,
early: bool,
hop: Option<HopNum>,
msg: UnparsedRelayMsg,
relay_cell_format: RelayCellFormat,
cell_counts_toward_windows: bool,
) -> StdResult<(), ReactorError>
async fn handle_relay_msg( &mut self, early: bool, hop: Option<HopNum>, msg: UnparsedRelayMsg, relay_cell_format: RelayCellFormat, cell_counts_toward_windows: bool, ) -> StdResult<(), ReactorError>
Handle a single incoming RELAY message.
Sourceasync fn handle_meta_msg(
&mut self,
early: bool,
hopnum: Option<HopNum>,
msg: UnparsedRelayMsg,
relay_cell_format: RelayCellFormat,
) -> StdResult<(), ReactorError>
async fn handle_meta_msg( &mut self, early: bool, hopnum: Option<HopNum>, msg: UnparsedRelayMsg, relay_cell_format: RelayCellFormat, ) -> StdResult<(), ReactorError>
Handle a RELAY or RELAY_EARLY message on this circuit with stream ID 0.
Sourceasync fn send_reactor_cmd(
&mut self,
forward: BackwardReactorCmd,
) -> StdResult<(), ReactorError>
async fn send_reactor_cmd( &mut self, forward: BackwardReactorCmd, ) -> StdResult<(), ReactorError>
Send a command to the backward reactor.
Blocks if the backward_reactor_tx channel is full, i.e. if the backward reactor
is not ready to send any more cells.
Returns an error if the backward reactor has shut down.