pub(crate) struct CircHop {
unique_id: TunnelScopedCircId,
hop_num: HopNum,
inbound: CircHopInbound,
outbound: CircHopOutbound,
}Expand description
Represents the reactor’s view of a single hop.
Fields
unique_id: TunnelScopedCircIdThe unique ID of the circuit. Used for logging.
hop_num: HopNumHop number in the path.
inbound: CircHopInboundThe inbound state of the hop.
Used for processing cells received from this hop.
outbound: CircHopOutboundThe outbound state of the hop.
Used for preparing cells to send to this hop.
Implementations
Sourceimpl CircHop
impl CircHop
Sourcepub(crate) fn new(
unique_id: TunnelScopedCircId,
hop_num: HopNum,
settings: &HopSettings,
) -> Self
pub(crate) fn new( unique_id: TunnelScopedCircId, hop_num: HopNum, settings: &HopSettings, ) -> Self
Create a new hop.
Sourcepub(crate) fn begin_stream(
&mut self,
message: AnyRelayMsg,
sender: StreamQueueSender,
rx: Receiver<AnyRelayMsg, MpscSpec>,
rate_limit_updater: Sender<StreamRateLimit>,
drain_rate_requester: NotifySender<DrainRateRequest>,
cmd_checker: Box<dyn CmdChecker + Send + 'static>,
) -> Result<(SendRelayCell, StreamId)>
pub(crate) fn begin_stream( &mut self, message: AnyRelayMsg, sender: StreamQueueSender, rx: Receiver<AnyRelayMsg, MpscSpec>, rate_limit_updater: Sender<StreamRateLimit>, drain_rate_requester: NotifySender<DrainRateRequest>, cmd_checker: Box<dyn CmdChecker + Send + 'static>, ) -> Result<(SendRelayCell, StreamId)>
Start a stream. Creates an entry in the stream map with the given channels, and sends the
message to the provided hop.
Sourcepub(crate) fn close_stream(
&mut self,
id: StreamId,
message: CloseStreamBehavior,
why: TerminateReason,
expiry: Instant,
) -> Result<Option<SendRelayCell>>
pub(crate) fn close_stream( &mut self, id: StreamId, message: CloseStreamBehavior, why: TerminateReason, expiry: Instant, ) -> Result<Option<SendRelayCell>>
Close the stream associated with id because the stream was
dropped.
Sourcepub(crate) fn maybe_send_xon(
&mut self,
rate: XonKbpsEwma,
id: StreamId,
) -> Result<Option<Xon>>
pub(crate) fn maybe_send_xon( &mut self, rate: XonKbpsEwma, id: StreamId, ) -> Result<Option<Xon>>
Check if we should send an XON message.
If we should, then returns the XON message that should be sent.
Sourcepub(crate) fn maybe_send_xoff(&mut self, id: StreamId) -> Result<Option<Xoff>>
pub(crate) fn maybe_send_xoff(&mut self, id: StreamId) -> Result<Option<Xoff>>
Check if we should send an XOFF message.
If we should, then returns the XOFF message that should be sent.
Sourcepub(crate) fn relay_cell_format(&self) -> RelayCellFormat
pub(crate) fn relay_cell_format(&self) -> RelayCellFormat
Return the format that is used for relay cells sent to this hop.
For the most part, this format isn’t necessary to interact with a CircHop; it becomes relevant when we are deciding what we can encode for the hop.
Sourcepub(crate) fn ccontrol(&self) -> MutexGuard<'_, CongestionControl>
pub(crate) fn ccontrol(&self) -> MutexGuard<'_, CongestionControl>
Return a mutable reference to our CongestionControl object.
Sourcepub(crate) fn outbound(&self) -> &CircHopOutbound
pub(crate) fn outbound(&self) -> &CircHopOutbound
Return a reference to our CircHopOutbound object.
Sourcepub(crate) fn about_to_send(
&mut self,
stream_id: StreamId,
msg: &AnyRelayMsg,
) -> Result<()>
pub(crate) fn about_to_send( &mut self, stream_id: StreamId, msg: &AnyRelayMsg, ) -> Result<()>
We’re about to send msg.
Sourcepub(crate) fn add_ent_with_id(
&self,
sink: StreamQueueSender,
rx: Receiver<AnyRelayMsg, MpscSpec>,
rate_limit_updater: Sender<StreamRateLimit>,
drain_rate_requester: NotifySender<DrainRateRequest>,
stream_id: StreamId,
cmd_checker: Box<dyn CmdChecker + Send + 'static>,
) -> Result<()>
pub(crate) fn add_ent_with_id( &self, sink: StreamQueueSender, rx: Receiver<AnyRelayMsg, MpscSpec>, rate_limit_updater: Sender<StreamRateLimit>, drain_rate_requester: NotifySender<DrainRateRequest>, stream_id: StreamId, cmd_checker: Box<dyn CmdChecker + Send + 'static>, ) -> Result<()>
Add an entry to this map using the specified StreamId.
Sourcepub(crate) fn ending_msg_received(&self, stream_id: StreamId) -> Result<()>
pub(crate) fn ending_msg_received(&self, stream_id: StreamId) -> Result<()>
Note that we received an END message (or other message indicating the end of
the stream) on the stream with id.
Sourcepub(crate) fn decode(
&mut self,
cell: BoxedCellBody,
) -> Result<RelayCellDecoderResult>
pub(crate) fn decode( &mut self, cell: BoxedCellBody, ) -> Result<RelayCellDecoderResult>
Parse a RELAY or RELAY_EARLY cell body.
Requires that the cryptographic checks on the message have already been performed
Sourcepub(super) fn handle_msg(
&self,
hop_detail: &PathEntry,
cell_counts_toward_windows: bool,
streamid: StreamId,
msg: UnparsedRelayMsg,
now: Instant,
) -> Result<Option<UnparsedRelayMsg>>
pub(super) fn handle_msg( &self, hop_detail: &PathEntry, cell_counts_toward_windows: bool, streamid: StreamId, msg: UnparsedRelayMsg, now: Instant, ) -> Result<Option<UnparsedRelayMsg>>
Handle msg, delivering it to the stream with the specified streamid if appropriate.
Returns back the provided msg, if the message is an incoming stream request
that needs to be handled by the calling code.
Sourcepub(crate) fn set_stream_map(
&mut self,
map: Arc<Mutex<StreamMap>>,
) -> StdResult<(), Bug>
pub(crate) fn set_stream_map( &mut self, map: Arc<Mutex<StreamMap>>, ) -> StdResult<(), Bug>
Set the stream map of this hop to map.
Returns an error if the existing stream map of the hop has any open stream.
Sourcepub(crate) fn decrement_outbound_cell_limit(&mut self) -> Result<()>
pub(crate) fn decrement_outbound_cell_limit(&mut self) -> Result<()>
Decrement the limit of outbound cells that may be sent to this hop; give an error if it would reach zero.
Sourcepub(crate) fn decrement_inbound_cell_limit(&mut self) -> Result<()>
pub(crate) fn decrement_inbound_cell_limit(&mut self) -> Result<()>
Decrement the limit of inbound cells that may be received from this hop; give an error if it would reach zero.