pub(crate) struct CircHopList {
hops: SmallVec<[CircHop; 3]>,
}Expand description
Represents the reactor’s view of a circuit’s hop.
Fields
hops: SmallVec<[CircHop; 3]>The list of hops.
Implementations
Sourceimpl CircHopList
impl CircHopList
Sourcepub(super) fn hop(&self, hopnum: HopNum) -> Option<&CircHop>
pub(super) fn hop(&self, hopnum: HopNum) -> Option<&CircHop>
Return a reference to the hop corresponding to hopnum, if there is one.
Sourcepub(super) fn get_mut(&mut self, hopnum: HopNum) -> Option<&mut CircHop>
pub(super) fn get_mut(&mut self, hopnum: HopNum) -> Option<&mut CircHop>
Return a mutable reference to the hop corresponding to hopnum, if there is one.
Sourcepub(in client::reactor) fn ready_streams_iterator(
&self,
exclude: Option<HopNum>,
) -> impl Stream<Item = CircuitCmd> + use<>
pub(in client::reactor) fn ready_streams_iterator( &self, exclude: Option<HopNum>, ) -> impl Stream<Item = CircuitCmd> + use<>
Returns a Stream of CircuitCmd to poll from the main loop.
The iterator contains at most one CircuitCmd for each hop,
representing the instructions for handling the ready-item, if any,
of its highest priority stream.
IMPORTANT: this stream locks the stream map mutexes of each CircHop!
To avoid contention, never create more than one
ready_streams_iterator
stream at a time!
This is cancellation-safe.
Sourcepub(super) fn remove_expired_halfstreams(&mut self, now: Instant)
pub(super) fn remove_expired_halfstreams(&mut self, now: Instant)
Remove all halfstreams that are expired at now.
Sourcepub(super) fn has_streams(&self) -> bool
pub(super) fn has_streams(&self) -> bool
Returns true if there are any streams on this circuit
Important: this function locks the stream map of its each of the CircHops
in this circuit, so it must not be called from any function where the
stream map lock is held (such as ready_streams_iterator.
Sourcepub(crate) fn tunnel_activity(&self) -> TunnelActivity
pub(crate) fn tunnel_activity(&self) -> TunnelActivity
Return the most active TunnelActivity for any hop on this CircHopList.