struct CellHandlers {
meta_handler: Option<Box<dyn MetaCellHandler + Send>>,
incoming_stream_req_handler: Option<IncomingStreamRequestHandler>,
}Expand description
Cell handlers, shared between the Reactor and its underlying Circuits.
Fields
meta_handler: Option<Box<dyn MetaCellHandler + Send>>A handler for a meta cell, together with a result channel to notify on completion.
NOTE(prop349): this is part of Arti’s “Base Circuit Hop Handler”.
Upon sending an EXTEND cell, the ControlHandler sets this handler
to CircuitExtender.
The handler is then used in Circuit::handle_meta_cell for handling
all the meta cells received on the circuit that are not SENDMEs or TRUNCATE
(which are handled separately) or conflux cells
(which are handled by the conflux handlers).
The handler is uninstalled after the receipt of the EXTENDED cell, so any subsequent EXTENDED cells will cause the circuit to be torn down.
incoming_stream_req_handler: Option<IncomingStreamRequestHandler>A handler for incoming stream requests.
Implementations
Sourceimpl CellHandlers
impl CellHandlers
Sourcefn set_meta_handler(
&mut self,
handler: Box<dyn MetaCellHandler + Send>,
) -> Result<()>
fn set_meta_handler( &mut self, handler: Box<dyn MetaCellHandler + Send>, ) -> Result<()>
Try to install a given meta-cell handler to receive any unusual cells on this circuit, along with a result channel to notify on completion.
Sourcefn set_incoming_stream_req_handler(
&mut self,
handler: IncomingStreamRequestHandler,
) -> Result<()>
fn set_incoming_stream_req_handler( &mut self, handler: IncomingStreamRequestHandler, ) -> Result<()>
Try to install a given cell handler on this circuit.