pub(crate) fn is_sendme_inc_valid(
inc: u8,
params: &CongestionControlParams,
) -> boolExpand description
Return true iff the given sendme increment is valid with regards to the value in the circuit parameters that is taken from the consensus.