pub(crate) type CircRecvWindow = RecvWindow<CircParams>;Expand description
A circuit’s receive window.
Aliased Type
pub(crate) struct CircRecvWindow {
window: u16,
_dummy: PhantomData<CircParams>,
}Fields
window: u16Number of cells that we’d be willing to receive on this window before sending a SENDME.
_dummy: PhantomData<CircParams>Marker type to tell the compiler that the P type is used.