pub(crate) type StreamRecvWindow = RecvWindow<StreamParams>;Expand description
A stream’s receive window.
Aliased Type
pub(crate) struct StreamRecvWindow {
window: u16,
_dummy: PhantomData<StreamParams>,
}Fields
window: u16Number of cells that we’d be willing to receive on this window before sending a SENDME.
_dummy: PhantomData<StreamParams>Marker type to tell the compiler that the P type is used.