pub struct CreateRequestHandler {
chan_provider: Weak<dyn ChannelProvider<BuildSpec = OwnedChanTarget> + Send + Sync>,
circ_net_params: RwLock<CircNetParameters>,
ntor_keys: RwLock<RelayNtorKeys>,
incoming_filter_factory: Box<dyn IncomingStreamRequestFilterFactory + Send + Sync>,
}Expand description
Everything needed to handle CREATE* messages on channels.
Fields
chan_provider: Weak<dyn ChannelProvider<BuildSpec = OwnedChanTarget> + Send + Sync>Something that can launch channels. Typically the ChanMgr.
circ_net_params: RwLock<CircNetParameters>Circuit-related network parameters.
ntor_keys: RwLock<RelayNtorKeys>The circuit extension keys.
incoming_filter_factory: Box<dyn IncomingStreamRequestFilterFactory + Send + Sync>An IncomingStreamRequestFilter factory for checking whether the user wants
this request, or wants to reject it immediately.
Used for obtaining a current IncomingStreamRequestFilter
for building a circuit reactor.
Implementations
Sourceimpl CreateRequestHandler
impl CreateRequestHandler
Sourcepub fn new(
chan_provider: Weak<dyn ChannelProvider<BuildSpec = OwnedChanTarget> + Send + Sync>,
circ_net_params: CircNetParameters,
ntor_keys: RelayNtorKeys,
incoming_filter_factory: Box<dyn IncomingStreamRequestFilterFactory + Send + Sync>,
) -> Self
pub fn new( chan_provider: Weak<dyn ChannelProvider<BuildSpec = OwnedChanTarget> + Send + Sync>, circ_net_params: CircNetParameters, ntor_keys: RelayNtorKeys, incoming_filter_factory: Box<dyn IncomingStreamRequestFilterFactory + Send + Sync>, ) -> Self
Build a new CreateRequestHandler.
Sourcepub fn update_params(&self, circ_net_params: CircNetParameters)
pub fn update_params(&self, circ_net_params: CircNetParameters)
Update the circuit parameters from a network consensus.
Sourcepub fn update_ntor_keys(&self, ntor_keys: RelayNtorKeys)
pub fn update_ntor_keys(&self, ntor_keys: RelayNtorKeys)
Update the handler with a new set of circuit extension keys.
This is called periodically by the relay key rotation task.
Sourcepub(crate) fn handle_create<R: Runtime>(
&self,
runtime: &R,
channel: &Arc<Channel>,
our_ed25519_id: &Ed25519Identity,
our_rsa_id: &RsaIdentity,
circ_id: CircId,
msg: &CreateRequest,
memquota: &ChannelAccount,
circ_unique_id: UniqId,
) -> Result<(CreateResponse, RelayCircComponents), Destroy>
pub(crate) fn handle_create<R: Runtime>( &self, runtime: &R, channel: &Arc<Channel>, our_ed25519_id: &Ed25519Identity, our_rsa_id: &RsaIdentity, circ_id: CircId, msg: &CreateRequest, memquota: &ChannelAccount, circ_unique_id: UniqId, ) -> Result<(CreateResponse, RelayCircComponents), Destroy>
Handle a CREATE* cell.
This intentionally does not return a crate::Error so that we don’t accidentally shut
down the channel reactor when we really should be returning a DESTROY. Shutting down a
channel may cause us to leak information about paths of circuits travelling through this
relay. This is especially important here since we’re handling data that is controllable from
the other end of the circuit.
Sourcefn handle_create_inner<R: Runtime>(
&self,
runtime: &R,
channel: &Arc<Channel>,
our_ed25519_id: &Ed25519Identity,
our_rsa_id: &RsaIdentity,
circ_id: CircId,
msg: &CreateRequest,
memquota: &ChannelAccount,
circ_unique_id: UniqId,
) -> Result<(CreateResponse, RelayCircComponents), HandleCreateError>
fn handle_create_inner<R: Runtime>( &self, runtime: &R, channel: &Arc<Channel>, our_ed25519_id: &Ed25519Identity, our_rsa_id: &RsaIdentity, circ_id: CircId, msg: &CreateRequest, memquota: &ChannelAccount, circ_unique_id: UniqId, ) -> Result<(CreateResponse, RelayCircComponents), HandleCreateError>
See Self::handle_create.
Sourcefn handle_create_fast(
&self,
msg: &CreateFast,
) -> Result<CompletedHandshakeComponents, HandleCreateError>
fn handle_create_fast( &self, msg: &CreateFast, ) -> Result<CompletedHandshakeComponents, HandleCreateError>
The handshake code for a CREATE_FAST request.
Sourcefn handle_create2_ntor(
&self,
msg_body: &[u8],
our_rsa_id: &RsaIdentity,
) -> Result<CompletedHandshakeComponents, HandleCreateError>
fn handle_create2_ntor( &self, msg_body: &[u8], our_rsa_id: &RsaIdentity, ) -> Result<CompletedHandshakeComponents, HandleCreateError>
The handshake code for a CREATE2 ntor (non-v3) request.
Sourcefn handle_create2_ntorv3(
&self,
_msg_body: &[u8],
_our_ed25519_id: &Ed25519Identity,
) -> Result<CompletedHandshakeComponents, HandleCreateError>
fn handle_create2_ntorv3( &self, _msg_body: &[u8], _our_ed25519_id: &Ed25519Identity, ) -> Result<CompletedHandshakeComponents, HandleCreateError>
The handshake code for a CREATE2 ntor-v3 request.
Sourcefn ntor_keys<T>(
&self,
map: impl FnMut(&RelayNtorKeypair) -> T,
) -> impl AsRef<[T]>
fn ntor_keys<T>( &self, map: impl FnMut(&RelayNtorKeypair) -> T, ) -> impl AsRef<[T]>
Helper to get the ntor keypairs after some transformation map.
The map transformation must be fast since it blocks a read lock.
The returned keys are sorted with the most recent key first.
It would be nice if this just returned an iterator, but the read lock prevents this.