pub struct RelayResponderHandshake<T: AsyncRead + AsyncWrite + CertifiedConn + StreamOps + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider> {
sleep_prov: S,
memquota: ChannelAccount,
framed_tls: Framed<T, ChannelCellHandler>,
peer_addr: Sensitive<PeerAddr>,
my_addrs: Vec<IpAddr>,
unique_id: UniqId,
auth_material: Arc<RelayChannelAuthMaterial>,
create_request_handler: Arc<CreateRequestHandler>,
}Expand description
A relay channel handshake as the responder.
Fields
sleep_prov: SRuntime handle (insofar as we need it)
memquota: ChannelAccountMemory quota account
framed_tls: Framed<T, ChannelCellHandler>Underlying TLS stream in a channel frame.
(We don’t enforce that this is actually TLS, but if it isn’t, the connection won’t be secure.)
peer_addr: Sensitive<PeerAddr>The peer IP address as in the address the initiator is connecting from. This can be a client so keep it sensitive.
my_addrs: Vec<IpAddr>Our advertised addresses. Needed for the NETINFO.
unique_id: UniqIdLogging identifier for this stream. (Used for logging only.)
auth_material: Arc<RelayChannelAuthMaterial>Our identity keys needed for authentication.
create_request_handler: Arc<CreateRequestHandler>Provided to each new channel so that they can handle CREATE* requests.
Implementations
Sourceimpl<T: AsyncRead + AsyncWrite + CertifiedConn + StreamOps + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider> RelayResponderHandshake<T, S>
impl<T: AsyncRead + AsyncWrite + CertifiedConn + StreamOps + Send + Unpin + 'static, S: CoarseTimeProvider + SleepProvider> RelayResponderHandshake<T, S>
Sourcepub(crate) fn new(
peer_addr: Sensitive<PeerAddr>,
my_addrs: Vec<SocketAddr>,
tls: T,
sleep_prov: S,
auth_material: Arc<RelayChannelAuthMaterial>,
memquota: ChannelAccount,
create_request_handler: Arc<CreateRequestHandler>,
) -> Self
pub(crate) fn new( peer_addr: Sensitive<PeerAddr>, my_addrs: Vec<SocketAddr>, tls: T, sleep_prov: S, auth_material: Arc<RelayChannelAuthMaterial>, memquota: ChannelAccount, create_request_handler: Arc<CreateRequestHandler>, ) -> Self
Constructor.
Sourcepub async fn handshake<F>(
self,
now_fn: F,
) -> Result<MaybeVerifiableRelayResponderChannel<T, S>>where
F: FnOnce() -> SystemTime,
pub async fn handshake<F>(
self,
now_fn: F,
) -> Result<MaybeVerifiableRelayResponderChannel<T, S>>where
F: FnOnce() -> SystemTime,
Begin the handshake process.
Takes a function that reports the current time. In theory, this can just be
SystemTime::get().
Sourceasync fn recv_cells_from_initiator(
&mut self,
) -> Result<(Option<(Certs, Authenticate, ClogDigest)>, (Netinfo, Instant))>
async fn recv_cells_from_initiator( &mut self, ) -> Result<(Option<(Certs, Authenticate, ClogDigest)>, (Netinfo, Instant))>
Receive all the cells expected from the initiator of the connection. Keep in mind that it can be either a relay or client or bridge.
Sourceasync fn send_cells_to_initiator(&mut self) -> Result<SlogDigest>
async fn send_cells_to_initiator(&mut self) -> Result<SlogDigest>
Send all expected cells to the initiator of the channel as the responder.
Return the SLOG (send log) digest to be later used when verifying the initiator’s AUTHENTICATE cell.