pub(crate) struct Reactor<R: Runtime>(Reactor<R, Forward, Backward>);Expand description
The entry point of the circuit reactor subsystem.
Tuple Fields
0: Reactor<R, Forward, Backward>Implementations
Sourceimpl<R: Runtime> Reactor<R>
impl<R: Runtime> Reactor<R>
Sourcepub(crate) fn new(
runtime: R,
channel: &Arc<Channel>,
circ_id: CircId,
unique_id: UniqId,
input: CircuitRxReceiver,
crypto_in: Box<dyn InboundRelayLayer + Send>,
crypto_out: Box<dyn OutboundRelayLayer + Send>,
settings: &HopSettings,
chan_provider: Arc<dyn ChannelProvider<BuildSpec = OwnedChanTarget> + Send + Sync>,
padding_ctrl: PaddingController,
padding_event_stream: PaddingEventStream,
incoming_filter: Box<dyn IncomingStreamRequestFilter>,
memquota: &CircuitAccount,
) -> Result<(Self, Arc<RelayCirc>, impl Stream<Item = IncomingStream> + use<R>)>
pub(crate) fn new( runtime: R, channel: &Arc<Channel>, circ_id: CircId, unique_id: UniqId, input: CircuitRxReceiver, crypto_in: Box<dyn InboundRelayLayer + Send>, crypto_out: Box<dyn OutboundRelayLayer + Send>, settings: &HopSettings, chan_provider: Arc<dyn ChannelProvider<BuildSpec = OwnedChanTarget> + Send + Sync>, padding_ctrl: PaddingController, padding_event_stream: PaddingEventStream, incoming_filter: Box<dyn IncomingStreamRequestFilter>, memquota: &CircuitAccount, ) -> Result<(Self, Arc<RelayCirc>, impl Stream<Item = IncomingStream> + use<R>)>
Create a new circuit reactor.
Returns the Reactor, a RelayCirc handle to it,
and a Stream of IncomingStreams.
The reactor will send outbound messages on channel, receive incoming
messages on input, and identify this circuit by the channel-local
CircId provided.
The internal unique identifier for this circuit will be unique_id.
The returned IncomingStreams are exit, dns, or directory streams.
An incoming stream is automatically rejected by the reactor
if the provided IncomingStreamRequestFilter rejects it.
You can also explicitly reject a stream by calling IncomingStream::reject.
If the Stream is dropped, the next incoming stream request
(BEGIN, BEGIN_DIR, or RESOLVE`)
on this circuit will cause the stream reactor to shut down,
which will trigger a shutdown of all the circuit reactors (FWD, BWD),
which causing the circuit to close.
The streams not rejected by the IncomingStreamRequestFilter will
get an entry in the circuit’s stream map.
Rejecting such a stream using IncomingStream::reject will remove the entry.
The IncomingStreamRequestFilter should only perform inexpensive checks
that won’t block the reactor.
More expensive, or blocking checks, should be handled outside of the circuit reactor,
when processing new IncomingStreams from the returned Rust stream.
Data and directory streams can be accepted by calling IncomingStream::accept_data.
The caller is responsible for proxying data between the resulting DataStream
and the local application stream.