pub(crate) trait BufferIsEmpty {
// Required method
fn is_empty(self: Pin<&mut Self>) -> bool;
}Expand description
Used by the XonXoffReader to decide when to send a drain rate update
(typically resulting in an XON message).
Required Methods
Sourcefn is_empty(self: Pin<&mut Self>) -> bool
fn is_empty(self: Pin<&mut Self>) -> bool
Returns true if there are no incoming bytes buffered on this stream.
This takes a &mut so that implementers can
unobtrusive_peek()
a stream if necessary.