struct SidechannelMitigation {
last_recvd_xon_xoff: Option<XonXoff>,
bytes_sent_total: Saturating<u64>,
num_advisory_xon_recvd: Saturating<u64>,
num_xoff_recvd: Saturating<u64>,
}Expand description
Sidechannel mitigations for DropMark attacks.
In order to mitigate DropMark attacks, both XOFF and advisory XON transmission must be restricted.
These restrictions should be implemented for clients (OPs and onion services).
Fields
last_recvd_xon_xoff: Option<XonXoff>The last rate limit update we received.
bytes_sent_total: Saturating<u64>Number of sent stream bytes.
C-tor has some logic to try to fit this into a 32-bit integer, but lets not do that unless we need to as it will make bugs more likely.
num_advisory_xon_recvd: Saturating<u64>The number of advisory XON messages we’ve received.
Note: Advisory XONs are XON->XON messages, and not XOFF->XON messages.
num_xoff_recvd: Saturating<u64>The number of XOFF messages we’ve received.
Implementations
Sourceimpl SidechannelMitigation
impl SidechannelMitigation
Sourcefn new() -> Self
fn new() -> Self
A new SidechannelMitigation.
Sourcefn peer_xoff_limit_bytes(params: &FlowCtrlParameters) -> u64
fn peer_xoff_limit_bytes(params: &FlowCtrlParameters) -> u64
A (likely underestimated) guess of the XOFF limit that the other endpoint is using.
Sourcefn peer_xon_limit_bytes(params: &FlowCtrlParameters) -> u64
fn peer_xon_limit_bytes(params: &FlowCtrlParameters) -> u64
A (likely underestimated) guess of the advisory XON limit that the other endpoint is using.
Sourcefn sent_stream_data(&mut self, stream_bytes: usize)
fn sent_stream_data(&mut self, stream_bytes: usize)
Notify that we have sent stream data.
Sourcefn received_xon(&mut self, params: &FlowCtrlParameters) -> Result<()>
fn received_xon(&mut self, params: &FlowCtrlParameters) -> Result<()>
Notify that we have received an XON message.
Sourcefn received_xoff(&mut self, params: &FlowCtrlParameters) -> Result<()>
fn received_xoff(&mut self, params: &FlowCtrlParameters) -> Result<()>
Notify that we have received an XOFF message.