type RelayCmdSet = BitArray<[usize; 4], Lsb0>;Expand description
Bit-vector used to represent a list of permitted commands.
This is cheaper and faster than using a vec, and avoids side-channel attacks.
Aliased Type
#[repr(transparent)]struct RelayCmdSet {
pub _ord: PhantomData<Lsb0>,
pub data: [usize; 4],
}Fields
_ord: PhantomData<Lsb0>The ordering of bits within an A::Store element.
data: [usize; 4]The wrapped data buffer.