pub(crate) struct PollAll<'a, const N: usize, T> {
inner: SmallVec<[Pin<Box<dyn Future<Output = T> + Send + 'a>>; N]>,
}Expand description
Helper for driving multiple futures in lockstep.
When .awaited, a PollAll will unconditionally poll all of its
underlying futures, in the order they were pushed,
until one or more of them resolves.
Any remaining unresolved futures will be dropped.
An empty PollAll will resolve immediately, yielding an empty list.
PollAll resolves to an ordered list of results, obtained from polling
the futures in insertion order. Because some of the futures may not
get a chance to resolve, the number of results will always
be less than or equal to the number of inserted futures.
Because PollAll drives the futures in lockstep,
if one future becomes ready, all of the futures will get polled,
even if they didn’t generate a wakeup notification.
Invariants
All of the futures inserted into this set must be cancellation safe.
Fields
inner: SmallVec<[Pin<Box<dyn Future<Output = T> + Send + 'a>>; N]>The futures to drive in lockstep.
Implementations