pub(crate) struct DynamicRateLimitedWriter<W: AsyncWrite, S, P: SleepProvider> {
writer: RateLimitedWriter<W, P>,
updates: S,
}Expand description
A rate-limited async writer.
This wraps a RateLimitedWriter and watches a stream for configuration changes (such as rate
limit changes).
Fields
writer: RateLimitedWriter<W, P>The rate-limited writer.
updates: SA stream that provides configuration updates, including rate limit updates.
Implementations
Sourceimpl<W: AsyncWrite, S, P: SleepProvider> DynamicRateLimitedWriter<W, S, P>
impl<W: AsyncWrite, S, P: SleepProvider> DynamicRateLimitedWriter<W, S, P>
Sourceimpl<W, S, P> DynamicRateLimitedWriter<W, S, P>where
W: AsyncWrite,
P: SleepProvider,
impl<W, S, P> DynamicRateLimitedWriter<W, S, P>where
W: AsyncWrite,
P: SleepProvider,
Sourcepub(crate) fn new(writer: RateLimitedWriter<W, P>, updates: S) -> Self
pub(crate) fn new(writer: RateLimitedWriter<W, P>, updates: S) -> Self
Create a new DynamicRateLimitedWriter.
This wraps the writer and watches for configuration changes from the updates stream.
Sourcepub(crate) fn inner(&self) -> &W
pub(crate) fn inner(&self) -> &W
Access the inner AsyncWrite writer of the RateLimitedWriter.