pub(crate) trait SinkExt<T>: Sink<T> {
// Provided method
fn poll_ready_unpin_bool(
&mut self,
cx: &mut Context<'_>,
) -> Result<bool, Self::Error>
where Self: Unpin { ... }
}Expand description
Extension trait for Sink
Provided Methods
Sourcefn poll_ready_unpin_bool(
&mut self,
cx: &mut Context<'_>,
) -> Result<bool, Self::Error>where
Self: Unpin,
fn poll_ready_unpin_bool(
&mut self,
cx: &mut Context<'_>,
) -> Result<bool, Self::Error>where
Self: Unpin,
Calls futures::Sink::poll_ready but requires Unpin and returns bool
Various gnarly places in the circuit reactor find this convenient.
TODO #1397 (circuit reactor) probably remove this when the circuit reactor is rewritten.
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".